from __future__ import unicode_literals

import unittest
from collections import namedtuple

from rest_framework import serializers
from rest_framework.response import Response
from rest_framework.test import APIRequestFactory, APITransactionTestCase, force_authenticate
from rest_framework.views import APIView

from waldur_core.core.fields import JsonField
from waldur_core.core.fields import TimestampField
from waldur_core.core.serializers import Base64Field, RestrictedSerializerMixin, GenericRelatedField
from waldur_core.core import utils
from waldur_core.logging.utils import get_loggable_models


class Base64Serializer(serializers.Serializer):
    content = Base64Field()


class Base64FieldTest(unittest.TestCase):
    def test_text_gets_base64_encoded_on_serialization(self):
        serializer = Base64Serializer(instance={'content': 'hello'})
        actual = serializer.data['content']

        self.assertEqual('aGVsbG8=', actual)

    def test_text_remains_base64_encoded_on_deserialization(self):
        serializer = Base64Serializer(data={'content': 'Zm9vYmFy'})

        self.assertTrue(serializer.is_valid())

        actual = serializer.validated_data['content']

        self.assertEqual('Zm9vYmFy', actual)

    def test_deserialization_fails_validation_on_incorrect_base64(self):
        serializer = Base64Serializer(data={'content': '***NOT-BASE-64***'})

        self.assertFalse(serializer.is_valid())
        self.assertIn('content', serializer.errors,
                      'There should be errors for content field')
        self.assertIn('This field should a be valid Base64 encoded string.',
                      serializer.errors['content'])


class GenericRelatedFieldTest(APITransactionTestCase):
    def setUp(self):
        from waldur_core.structure.tests.factories import UserFactory

        self.user = UserFactory(is_staff=True)
        self.request = APIRequestFactory().get('/')
        self.request.user = self.user

        self.field = GenericRelatedField(related_models=get_loggable_models())
        self.field.root._context = {'request': self.request}

    def test_if_related_object_exists_it_is_deserialized(self):
        from waldur_core.structure.tests.factories import CustomerFactory
        customer = CustomerFactory()
        valid_url = CustomerFactory.get_url(customer)
        self.assertEqual(self.field.to_internal_value(valid_url), customer)

    def test_if_related_object_does_not_exist_validation_error_is_raised(self):
        from waldur_core.structure.tests.factories import CustomerFactory
        customer = CustomerFactory()
        valid_url = CustomerFactory.get_url(customer)
        customer.delete()
        self.assertRaises(serializers.ValidationError, self.field.to_internal_value, valid_url)

    def test_if_user_does_not_have_permissions_for_related_object_validation_error_is_raised(self):
        from waldur_core.structure.tests.factories import CustomerFactory
        customer = CustomerFactory()
        valid_url = CustomerFactory.get_url(customer)
        self.user.is_staff = False
        self.user.save()
        self.assertRaises(serializers.ValidationError, self.field.to_internal_value, valid_url)

    def test_if_uuid_is_invalid_validation_error_is_raised(self):
        invalid_url = 'https://example.com/api/customers/invalid/'
        self.assertRaises(serializers.ValidationError, self.field.to_internal_value, invalid_url)


class JsonSerializer(serializers.Serializer):
    content = JsonField()


class JsonFieldTest(unittest.TestCase):
    def test_dict_gets_parsed_as_dict_on_serialization(self):
        serializer = JsonSerializer(instance={'content': {u'key': u'value'}})
        actual = serializer.data['content']

        self.assertEqual({u'key': u'value'}, actual)

    def test_text_gets_json_parsed_on_deserialization(self):
        serializer = JsonSerializer(data={'content': '{"key": "value"}'})

        self.assertTrue(serializer.is_valid())

        actual = serializer.validated_data['content']

        self.assertEqual({u'key': u'value'}, actual)

    def test_deserialization_fails_validation_on_incorrect_json(self):
        serializer = JsonSerializer(data={'content': '***NOT-JSON***'})

        self.assertFalse(serializer.is_valid())
        self.assertIn('content', serializer.errors,
                      'There should be errors for content field')
        self.assertIn('This field should a be valid JSON string.',
                      serializer.errors['content'])


class TimestampSerializer(serializers.Serializer):
    content = TimestampField()


class TimestampFieldTest(unittest.TestCase):
    def setUp(self):
        self.datetime = utils.timeshift(days=-1)
        self.timestamp = utils.datetime_to_timestamp(self.datetime)

    def test_datetime_serialized_as_timestamp(self):
        serializer = TimestampSerializer(instance={'content': self.datetime})
        actual = serializer.data['content']
        self.assertEqual(self.timestamp, actual)

    def test_timestamp_parsed_as_datetime(self):
        serializer = TimestampSerializer(data={'content': str(self.timestamp)})
        self.assertTrue(serializer.is_valid())
        actual = serializer.validated_data['content']
        self.assertEqual(self.datetime, actual)

    def test_incorrect_timestamp(self):
        serializer = TimestampSerializer(data={'content': 'NOT_A_UNIX_TIMESTAMP'})
        self.assertFalse(serializer.is_valid())
        self.assertIn('content', serializer.errors,
                      'There should be errors for content field')
        self.assertIn('Value "NOT_A_UNIX_TIMESTAMP" should be valid UNIX timestamp.',
                      serializer.errors['content'])


class RestrictedSerializer(RestrictedSerializerMixin, serializers.Serializer):
    name = serializers.ReadOnlyField()
    url = serializers.ReadOnlyField()
    id = serializers.ReadOnlyField()


class RestrictedSerializerView(APIView):
    def get(self, request):
        User = namedtuple('User', ('name', 'url', 'id'))
        user = User(name='Walter', url='http://example.com/Walter', id=1)
        serializer = RestrictedSerializer(user, context={'request': request})
        return Response(serializer.data)


class RestrictedSerializerTest(APITransactionTestCase):
    def test_serializer_returns_fields_required_in_request(self):
        fields = ['name', 'url']
        response = self.make_request(fields)
        self.assertEqual(fields, response.data.keys())

    def make_request(self, fields):
        from waldur_core.structure.tests.factories import UserFactory

        request = APIRequestFactory().get('/', {'field': fields})
        force_authenticate(request, UserFactory())
        response = RestrictedSerializerView.as_view()(request)
        return response



import unittest

from rest_framework import test, status

from waldur_core.core import models as core_models
from waldur_core.structure.models import NewResource, ServiceSettings
from waldur_core.structure.tests import factories, fixtures, models as test_models


States = core_models.StateMixin.States


class ResourceRemovalTest(test.APITransactionTestCase):
    def setUp(self):
        self.user = factories.UserFactory(is_staff=True)
        self.client.force_authenticate(user=self.user)

    @unittest.skip('Unlink operation is not supported for new style resources yet.')
    def test_vm_unlinked_immediately_anyway(self):
        vm = factories.TestNewInstanceFactory(state=States.UPDATING)
        url = factories.TestNewInstanceFactory.get_url(vm, 'unlink')
        response = self.client.post(url)
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT, response.data)

    @unittest.skip('Unlink operation is not supported for new style resources yet.')
    def test_new_resource_unlinked_immediately(self):
        vm = factories.TestNewInstanceFactory(state=NewResource.States.OK)
        url = factories.TestNewInstanceFactory.get_url(vm, 'unlink')

        response = self.client.post(url)

        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT, response.data)

    def test_when_virtual_machine_is_deleted_descendant_resources_unlinked(self):
        # Arrange
        vm = factories.TestNewInstanceFactory()
        settings = factories.ServiceSettingsFactory(scope=vm)
        service = factories.TestServiceFactory(settings=settings)
        link = factories.TestServiceProjectLinkFactory(service=service)
        child_vm = factories.TestNewInstanceFactory(service_project_link=link)
        other_vm = factories.TestNewInstanceFactory()

        # Act
        vm.delete()

        # Assert
        self.assertFalse(test_models.TestNewInstance.objects.filter(id=child_vm.id).exists())
        self.assertFalse(test_models.TestService.objects.filter(id=service.id).exists())
        self.assertFalse(ServiceSettings.objects.filter(id=settings.id).exists())
        self.assertTrue(test_models.TestNewInstance.objects.filter(id=other_vm.id).exists())


class ResourceCreateTest(test.APITransactionTestCase):

    def setUp(self):
        self.fixture = fixtures.ServiceFixture()

    def test_resource_cannot_be_created_for_invalid_service_project_link(self):
        self.fixture.project.certifications.add(factories.ServiceCertificationFactory())
        self.assertFalse(self.fixture.service_project_link.is_valid)

        payload = {
            'service_project_link': factories.TestServiceProjectLinkFactory.get_url(self.fixture.service_project_link),
            'name': 'impossible resource',
        }
        url = factories.TestNewInstanceFactory.get_list_url()

        self.client.force_authenticate(user=self.fixture.staff)
        response = self.client.post(url, payload)

        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertIn('service_project_link', response.data)

    def test_shared_key_is_valid_for_virtual_machine_serializer(self):
        shared_key = factories.SshPublicKeyFactory(is_shared=True)
        key_url = factories.SshPublicKeyFactory.get_url(shared_key)

        spl_url = factories.TestServiceProjectLinkFactory.get_url(self.fixture.service_project_link)
        payload = {
            'service_project_link': spl_url,
            'name': 'valid name',
            'ssh_public_key': key_url,
        }
        url = factories.TestNewInstanceFactory.get_list_url()

        self.client.force_authenticate(user=self.fixture.owner)
        response = self.client.post(url, payload)
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)


class ResourceTagsTest(test.APITransactionTestCase):
    def setUp(self):
        self.fixture = fixtures.ServiceFixture()
        self.client.force_authenticate(user=self.fixture.staff)

    def test_resource_tags_are_rendered_as_list(self):
        self.fixture.resource.tags.add('tag1')
        self.fixture.resource.tags.add('tag2')

        url = factories.TestNewInstanceFactory.get_url(self.fixture.resource)
        response = self.client.get(url)
        self.assertEqual(response.data['tags'], ['tag1', 'tag2'])

    def test_tags_are_saved_on_resource_provision(self):
        payload = {
            'service_project_link': factories.TestServiceProjectLinkFactory.get_url(self.fixture.service_project_link),
            'name': 'Tagged resource',
            'tags': ['tag1', 'tag2']
        }
        url = factories.TestNewInstanceFactory.get_list_url()

        response = self.client.post(url, payload)
        self.assertEqual(response.status_code, status.HTTP_201_CREATED, response.data)
        self.assertItemsEqual(response.data['tags'], ['tag1', 'tag2'])

    def test_tags_are_saved_on_resource_modification(self):
        resource = self.fixture.resource
        resource.state = test_models.TestNewInstance.States.OK
        resource.save()
        resource.tags.add('tag1')
        resource.tags.add('tag2')
        payload = {'tags': []}
        url = factories.TestNewInstanceFactory.get_url(resource)

        response = self.client.patch(url, payload)
        self.assertEqual(response.status_code, status.HTTP_200_OK, response.data)
        self.assertEqual(response.data['tags'], [])

    def test_resource_can_be_filtered_by_tag(self):
        self.fixture.resource.tags.add('tag1')
        resource2 = factories.TestNewInstanceFactory(service_project_link=self.fixture.service_project_link)
        resource2.tags.add('tag2')

        url = factories.TestNewInstanceFactory.get_list_url()
        response = self.client.get(url, {'tag': 'tag1'})
        self.assertEqual(len(response.data), 1)



from __future__ import unicode_literals

import collections
import datetime
import mock

from django.contrib.auth import get_user_model
from django.utils import timezone
from rest_framework import status, test
from rest_framework.reverse import reverse

from waldur_core.core.tests.helpers import override_waldur_core_settings
from waldur_core.structure import tasks
from waldur_core.structure.models import CustomerRole, ProjectRole, CustomerPermission
from waldur_core.structure.tests import factories

User = get_user_model()

TestRole = collections.namedtuple('TestRole', ['user', 'customer', 'role'])


class CustomerPermissionBaseTest(test.APITransactionTestCase):
    all_roles = (
        # user customer role
        TestRole('first', 'first', 'owner'),
        TestRole('second', 'second', 'owner'),
    )

    role_map = {
        'owner': CustomerRole.OWNER,
    }

    def setUp(self):
        self.users = {
            'staff': factories.UserFactory(is_staff=True),
            'first': factories.UserFactory(),
            'first_manager': factories.UserFactory(),
            'first_admin': factories.UserFactory(),
            'second': factories.UserFactory(),
            'no_role': factories.UserFactory(),
        }

        self.customers = {
            'first': factories.CustomerFactory(),
            'second': factories.CustomerFactory(),
        }

        customer = self.customers['first']
        project = factories.ProjectFactory(customer=customer)

        for user, customer, role in self.all_roles:
            self.customers[customer].add_user(self.users[user], self.role_map[role])

        project.add_user(self.users['first_admin'], ProjectRole.ADMINISTRATOR)

    # Helper methods
    def _get_permission_url(self, user, customer, role):
        permission = CustomerPermission.objects.get(
            user=self.users[user],
            role=self.role_map[role],
            customer=self.customers[customer],
        )
        return 'http://testserver' + reverse('customer_permission-detail', kwargs={'pk': permission.pk})


class CustomerPermissionListTest(CustomerPermissionBaseTest):

    def test_anonymous_user_cannot_list_customer_permissions(self):
        response = self.client.get(reverse('customer_permission-list'))
        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)

    def test_user_cannot_list_roles_of_customer_he_is_not_affiliated(self):
        self.assert_user_access_to_permission_list(user='no_role', customer='first', should_see=False)
        self.assert_user_access_to_permission_list(user='no_role', customer='second', should_see=False)

    def test_customer_owner_can_list_roles_of_his_customer(self):
        self.assert_user_access_to_permission_list(user='first', customer='first', should_see=True)

    def test_project_admin_can_list_roles_of_his_customer(self):
        self.assert_user_access_to_permission_list(user='first_admin', customer='first', should_see=True)

    def test_staff_can_list_roles_of_any_customer(self):
        self.assert_user_access_to_permission_list(user='staff', customer='first', should_see=True)
        self.assert_user_access_to_permission_list(user='staff', customer='second', should_see=True)

    def test_customer_owner_cannot_list_roles_of_another_customer(self):
        self.assert_user_access_to_permission_list(user='first', customer='second', should_see=False)

    def test_project_admin_cannot_list_roles_of_another_customer(self):
        self.assert_user_access_to_permission_list(user='first_admin', customer='second', should_see=False)

    def assert_user_access_to_permission_list(self, user, customer, should_see):
        self.client.force_authenticate(user=self.users[user])

        response = self.client.get(reverse('customer_permission-list'))
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        expected_urls = {
            r: self._get_permission_url(*r)
            for r in self.all_roles
            if r.customer == customer
        }

        actual_urls = set([role['url'] for role in response.data])

        for role, role_url in expected_urls.items():
            if should_see:
                self.assertIn(
                    role_url, actual_urls,
                    '{0} user does not see privilege '
                    'he is supposed to see: {1}'.format(user, role),
                )
            else:
                self.assertNotIn(
                    role_url, actual_urls,
                    '{0} user sees privilege '
                    'he is not supposed to see: {1}'.format(user, role),
                )


class CustomerPermissionGrantTest(CustomerPermissionBaseTest):
    @override_waldur_core_settings(OWNERS_CAN_MANAGE_OWNERS=True)
    def test_customer_owner_can_grant_new_role_within_his_customer(self):
        self.assert_user_access_to_permission_granting(
            login_user='first',
            affected_user='no_role',
            affected_customer='first',
            expected_status=status.HTTP_201_CREATED,
        )

    def test_customer_owner_cannot_grant_existing_role_within_his_customer(self):
        self.assert_user_access_to_permission_granting(
            login_user='staff',
            affected_user='first',
            affected_customer='first',
            expected_status=status.HTTP_400_BAD_REQUEST,
            expected_payload={
                'non_field_errors': ['The fields customer and user must make a unique set.'],
            }
        )

    def test_customer_owner_cannot_grant_role_within_another_customer(self):
        self.assert_user_access_to_permission_granting(
            login_user='first',
            affected_user='no_role',
            affected_customer='second',
            expected_status=status.HTTP_400_BAD_REQUEST,
            expected_payload={
                'customer': ['Invalid hyperlink - Object does not exist.'],
            }
        )

    @override_waldur_core_settings(OWNERS_CAN_MANAGE_OWNERS=False)
    def test_customer_owner_can_not_grant_new_role_within_his_customer_if_settings_are_tweaked(self):
        self.assert_user_access_to_permission_granting(
            login_user='first',
            affected_user='no_role',
            affected_customer='first',
            expected_status=status.HTTP_403_FORBIDDEN,
        )

    @override_waldur_core_settings(OWNERS_CAN_MANAGE_OWNERS=True)
    def test_project_admin_cannot_grant_role_within_his_customer(self):
        self.assert_user_access_to_permission_granting(
            login_user='first_admin',
            affected_user='no_role',
            affected_customer='first',
            expected_status=status.HTTP_403_FORBIDDEN,
            expected_payload={
                'detail': 'You do not have permission to perform this action.',
            }
        )

    def test_staff_can_grant_new_role_within_any_customer(self):
        self.assert_user_access_to_permission_granting(
            login_user='staff',
            affected_user='no_role',
            affected_customer='first',
            expected_status=status.HTTP_201_CREATED,
        )
        self.assert_user_access_to_permission_granting(
            login_user='staff',
            affected_user='no_role',
            affected_customer='second',
            expected_status=status.HTTP_201_CREATED,
        )

    def test_staff_cannot_grant_permission_if_customer_quota_exceeded(self):
        affected_customer = 'first'
        self.customers[affected_customer].set_quota_limit('nc_user_count', 0)

        self.assert_user_access_to_permission_granting(
            login_user='staff',
            affected_user='no_role',
            affected_customer=affected_customer,
            expected_status=status.HTTP_409_CONFLICT,
        )

    def test_staff_cannot_grant_existing_role_within_any_customer(self):
        self.assert_user_access_to_permission_granting(
            login_user='staff',
            affected_user='first',
            affected_customer='first',
            expected_status=status.HTTP_400_BAD_REQUEST,
            expected_payload={
                'non_field_errors': ['The fields customer and user must make a unique set.'],
            }
        )
        self.assert_user_access_to_permission_granting(
            login_user='staff',
            affected_user='second',
            affected_customer='second',
            expected_status=status.HTTP_400_BAD_REQUEST,
            expected_payload={
                'non_field_errors': ['The fields customer and user must make a unique set.'],
            }
        )

    def assert_user_access_to_permission_granting(self, login_user, affected_user, affected_customer,
                                                  expected_status, expected_payload=None):
        self.client.force_authenticate(user=self.users[login_user])

        data = {
            'customer': factories.CustomerFactory.get_url(self.customers[affected_customer]),
            'user': factories.UserFactory.get_url(self.users[affected_user]),
            'role': 'owner',
        }

        response = self.client.post(reverse('customer_permission-list'), data)
        self.assertEqual(response.status_code, expected_status)
        if expected_payload is not None:
            self.assertDictContainsSubset(expected_payload, response.data)


class CustomerPermissionRevokeTest(CustomerPermissionBaseTest):
    @override_waldur_core_settings(OWNERS_CAN_MANAGE_OWNERS=True)
    def test_customer_owner_can_revoke_role_within_his_customer(self):
        self.assert_user_access_to_permission_revocation(
            login_user='first',
            affected_user='first',
            affected_customer='first',
            expected_status=status.HTTP_204_NO_CONTENT,
        )

    def test_customer_owner_cannot_revoke_role_within_another_customer(self):
        self.assert_user_access_to_permission_revocation(
            login_user='first',
            affected_user='second',
            affected_customer='second',
            expected_status=status.HTTP_404_NOT_FOUND,
        )

    @override_waldur_core_settings(OWNERS_CAN_MANAGE_OWNERS=False)
    def test_customer_owner_can_not_revoke_role_within_his_customer_if_settings_are_tweaked(self):
        self.assert_user_access_to_permission_revocation(
            login_user='first',
            affected_user='first',
            affected_customer='first',
            expected_status=status.HTTP_403_FORBIDDEN,
        )

    @override_waldur_core_settings(OWNERS_CAN_MANAGE_OWNERS=True)
    def test_project_admin_cannot_revoke_role_within_his_customer(self):
        self.assert_user_access_to_permission_revocation(
            login_user='first_admin',
            affected_user='first',
            affected_customer='first',
            expected_status=status.HTTP_403_FORBIDDEN,
            expected_payload={
                'detail': 'You do not have permission to perform this action.',
            }
        )

    def test_staff_can_revoke_role_within_any_customer(self):
        self.assert_user_access_to_permission_revocation(
            login_user='staff',
            affected_user='first',
            affected_customer='first',
            expected_status=status.HTTP_204_NO_CONTENT,
        )
        self.assert_user_access_to_permission_revocation(
            login_user='staff',
            affected_user='second',
            affected_customer='second',
            expected_status=status.HTTP_204_NO_CONTENT,
        )

    def assert_user_access_to_permission_revocation(self, login_user, affected_user, affected_customer,
                                                    expected_status, expected_payload=None):
        self.client.force_authenticate(user=self.users[login_user])

        url = self._get_permission_url(affected_user, affected_customer, 'owner')

        response = self.client.delete(url)
        self.assertEqual(response.status_code, expected_status)
        if expected_payload is not None:
            self.assertDictContainsSubset(expected_payload, response.data)


class CustomerPermissionFilterTest(test.APITransactionTestCase):
    def setUp(self):
        staff_user = factories.UserFactory(is_staff=True)
        self.client.force_authenticate(user=staff_user)

        self.users = {
            'first': factories.UserFactory(),
            'second': factories.UserFactory(),
        }

        self.customers = {
            'first': factories.CustomerFactory(),
            'second': factories.CustomerFactory(),
        }

        for customer in self.customers:
            self.customers[customer].add_user(self.users['first'], CustomerRole.OWNER)
            self.customers[customer].add_user(self.users['second'], CustomerRole.OWNER)

    def test_staff_user_can_filter_roles_within_customer_by_customer_uuid(self):
        response = self.client.get(reverse('customer_permission-list'))
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        for customer in self.customers:
            response = self.client.get(reverse('customer_permission-list'),
                                       data={'customer': self.customers[customer].uuid})
            self.assertEqual(response.status_code, status.HTTP_200_OK)

            customer_url = self._get_customer_url(self.customers[customer])

            for permission in response.data:
                self.assertEqual(customer_url, permission['customer'])

    def test_staff_user_can_filter_roles_within_customer_by_username(self):
        response = self.client.get(reverse('customer_permission-list'))
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        for user in self.users:
            self._ensure_matching_entries_in('username', self.users[user].username)
            self._ensure_non_matching_entries_not_in('username', self.users[user].username)

    def test_staff_user_can_filter_roles_within_customer_by_native_name(self):
        response = self.client.get(reverse('customer_permission-list'))
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        for user in self.users:
            self._ensure_matching_entries_in('native_name', self.users[user].native_name)
            self._ensure_non_matching_entries_not_in('native_name', self.users[user].native_name)

    def test_staff_user_can_filter_roles_within_customer_by_full_name(self):
        response = self.client.get(reverse('customer_permission-list'))
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        for user in self.users:
            self._ensure_matching_entries_in('full_name', self.users[user].full_name)
            self._ensure_non_matching_entries_not_in('full_name', self.users[user].full_name)

    def test_staff_user_can_filter_roles_within_customer_by_role_name(self):
        response = self.client.get(reverse('customer_permission-list'))
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        response = self.client.get(reverse('customer_permission-list'),
                                   data={'role': 'owner'})
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        for permission in response.data:
            self.assertEqual('owner', permission['role'])

    def test_staff_user_cannot_filter_roles_within_customer_by_role_pk(self):
        response = self.client.get(reverse('customer_permission-list'))
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        response = self.client.get(reverse('customer_permission-list'),
                                   data={'role': '1'})
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data, [])

    def test_staff_user_can_see_required_fields_in_filtration_response(self):
        response = self.client.get(reverse('customer_permission-list'))
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        for customer in self.customers:
            response = self.client.get(reverse('customer_permission-list'),
                                       data={'customer': self.customers[customer].uuid})
            self.assertEqual(response.status_code, status.HTTP_200_OK)

            required_fields = ('url', 'user_native_name', 'user_full_name', 'user_username')

            for permission in response.data:
                for field in required_fields:
                    self.assertIn(field, permission)

    # Helper methods
    def _ensure_matching_entries_in(self, field, value):
        response = self.client.get(reverse('customer_permission-list'),
                                   data={field: value})
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        for permission in response.data:
                self.assertEqual(value, permission['user_' + field])

    def _ensure_non_matching_entries_not_in(self, field, value):
        user = factories.UserFactory()

        customer = factories.CustomerFactory()
        customer.add_user(user, CustomerRole.OWNER)

        response = self.client.get(reverse('customer_permission-list'),
                                   data={field: getattr(user, field)})
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        for permission in response.data:
                self.assertNotEqual(value, permission['user_' + field])

    def _get_customer_url(self, customer):
        return 'http://testserver' + reverse('customer-detail', kwargs={'uuid': customer.uuid})


class CustomerPermissionExpirationTest(test.APITransactionTestCase):
    def setUp(self):
        permission = factories.CustomerPermissionFactory()
        self.user = permission.user
        self.customer = permission.customer
        self.url = factories.CustomerPermissionFactory.get_url(permission)

    def test_user_can_not_update_permission_expiration_time_for_himself(self):
        self.client.force_authenticate(user=self.user)

        expiration_time = timezone.now() + datetime.timedelta(days=100)
        response = self.client.put(self.url, {
            'expiration_time': expiration_time
        })
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_staff_can_update_permission_expiration_time_for_any_user(self):
        staff_user = factories.UserFactory(is_staff=True)
        self.client.force_authenticate(user=staff_user)

        expiration_time = timezone.now() + datetime.timedelta(days=100)
        response = self.client.put(self.url, {
            'expiration_time': expiration_time
        })
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data['expiration_time'], expiration_time, response.data)

    @override_waldur_core_settings(OWNERS_CAN_MANAGE_OWNERS=True)
    def test_owner_can_update_permission_expiration_time_for_other_owner_in_same_customer(self):
        owner = factories.UserFactory()
        self.customer.add_user(owner, CustomerRole.OWNER)
        self.client.force_authenticate(user=owner)

        expiration_time = timezone.now() + datetime.timedelta(days=100)
        response = self.client.put(self.url, {
            'expiration_time': expiration_time
        })
        self.assertEqual(response.status_code, status.HTTP_200_OK, response.data)
        self.assertEqual(response.data['expiration_time'], expiration_time, response.data)

    @override_waldur_core_settings(OWNERS_CAN_MANAGE_OWNERS=False)
    def test_owner_can_not_update_permission_expiration_time_for_other_owner_if_settings_are_tweaked(self):
        owner = factories.UserFactory()
        self.customer.add_user(owner, CustomerRole.OWNER)
        self.client.force_authenticate(user=owner)

        expiration_time = timezone.now() + datetime.timedelta(days=100)
        response = self.client.put(self.url, {
            'expiration_time': expiration_time
        })
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_user_can_not_set_permission_expiration_time_lower_than_current(self):
        staff_user = factories.UserFactory(is_staff=True)
        self.client.force_authenticate(user=staff_user)

        expiration_time = timezone.now() - datetime.timedelta(days=100)
        response = self.client.put(self.url, {
            'expiration_time': expiration_time
        })
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    def test_user_cannot_grant_permissions_with_greater_expiration_time(self):
        expiration_time = timezone.now() + datetime.timedelta(days=100)
        permission = factories.CustomerPermissionFactory(expiration_time=expiration_time)
        self.client.force_authenticate(user=permission.user)
        response = self.client.post(factories.CustomerPermissionFactory.get_list_url(), {
            'customer': factories.CustomerFactory.get_url(customer=permission.customer),
            'user': factories.UserFactory.get_url(),
            'role': CustomerRole.OWNER,
            'expiration_time': expiration_time + datetime.timedelta(days=1),
        })
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_user_can_set_expiration_time_when_role_is_created(self):
        staff_user = factories.UserFactory(is_staff=True)
        self.client.force_authenticate(user=staff_user)

        expiration_time = timezone.now() + datetime.timedelta(days=100)
        response = self.client.post(factories.CustomerPermissionFactory.get_list_url(), {
            'customer': factories.CustomerFactory.get_url(),
            'user': factories.UserFactory.get_url(),
            'role': factories.CustomerPermissionFactory.role,
            'expiration_time': expiration_time,
        })
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
        self.assertEqual(response.data['expiration_time'], expiration_time, response.data)

    def test_task_revokes_expired_permissions(self):
        expired_permission = factories.CustomerPermissionFactory(
            expiration_time=timezone.now() - datetime.timedelta(days=100))
        not_expired_permission = factories.CustomerPermissionFactory(
            expiration_time=timezone.now() + datetime.timedelta(days=100))
        tasks.check_expired_permissions()

        self.assertFalse(expired_permission.customer.has_user(
            expired_permission.user, expired_permission.role))
        self.assertTrue(not_expired_permission.customer.has_user(
            not_expired_permission.user, not_expired_permission.role))

    def test_when_expiration_time_is_updated_event_is_emitted(self):
        staff_user = factories.UserFactory(is_staff=True)
        self.client.force_authenticate(user=staff_user)
        expiration_time = timezone.now() + datetime.timedelta(days=100)

        with mock.patch('logging.LoggerAdapter.info') as mocked_info:
            self.client.put(self.url, {'expiration_time': expiration_time})
            (args, kwargs) = mocked_info.call_args_list[-1]
            event_type = kwargs['extra']['event_type']
            event_message = args[0]
            self.assertEqual(event_type, 'role_updated')
            self.assertTrue(staff_user.full_name in event_message)


class CustomerPermissionCreatedByTest(test.APITransactionTestCase):
    def test_user_which_granted_permission_is_stored(self):
        staff_user = factories.UserFactory(is_staff=True)
        self.client.force_authenticate(user=staff_user)

        user = factories.UserFactory()
        customer = factories.CustomerFactory()

        data = {
            'customer': factories.CustomerFactory.get_url(customer),
            'user': factories.UserFactory.get_url(user),
            'role': CustomerRole.OWNER,
        }

        response = self.client.post(reverse('customer_permission-list'), data)
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

        permission = CustomerPermission.objects.get(pk=response.data['pk'])
        self.assertEqual(permission.created_by, staff_user)



from ddt import ddt, data

from rest_framework import status, test

from waldur_core.core.tests.helpers import override_waldur_core_settings
from waldur_core.structure import models

from . import fixtures, factories


class ServiceSettingsListTest(test.APITransactionTestCase):
    def setUp(self):
        self.users = {
            'staff': factories.UserFactory(is_staff=True),
            'owner': factories.UserFactory(),
            'not_owner': factories.UserFactory(),
        }

        self.customers = {
            'owned': factories.CustomerFactory(),
            'inaccessible': factories.CustomerFactory(),
        }

        self.customers['owned'].add_user(self.users['owner'], models.CustomerRole.OWNER)

        self.settings = {
            'shared': factories.ServiceSettingsFactory(shared=True),
            'inaccessible': factories.ServiceSettingsFactory(customer=self.customers['inaccessible']),
            'owned': factories.ServiceSettingsFactory(
                customer=self.customers['owned'], backend_url='bk.url', password='123'),
        }

        # Token is excluded, because it is not available for OpenStack
        self.credentials = ('backend_url', 'username', 'password')

    def test_user_can_see_shared_settings(self):
        self.client.force_authenticate(user=self.users['not_owner'])

        response = self.client.get(factories.ServiceSettingsFactory.get_list_url())
        self.assertEqual(response.status_code, status.HTTP_200_OK, response.data)
        self.assertEqual(len(response.data), 1)
        self.assert_credentials_hidden(response.data[0])
        self.assertEqual(response.data[0]['uuid'], self.settings['shared'].uuid.hex, response.data)

    def test_user_can_see_shared_and_own_settings(self):
        self.client.force_authenticate(user=self.users['owner'])

        response = self.client.get(factories.ServiceSettingsFactory.get_list_url())
        uuids_recieved = [d['uuid'] for d in response.data]
        uuids_expected = [self.settings[s].uuid.hex for s in ('shared', 'owned')]
        self.assertItemsEqual(uuids_recieved, uuids_expected, response.data)

    def test_admin_can_see_all_settings(self):
        self.client.force_authenticate(user=self.users['staff'])

        response = self.client.get(factories.ServiceSettingsFactory.get_list_url())
        uuids_recieved = [d['uuid'] for d in response.data]
        uuids_expected = [s.uuid.hex for s in self.settings.values()]
        self.assertItemsEqual(uuids_recieved, uuids_expected, uuids_recieved)

    def test_user_can_see_credentials_of_own_settings(self):
        self.client.force_authenticate(user=self.users['owner'])

        response = self.client.get(factories.ServiceSettingsFactory.get_url(self.settings['owned']))
        self.assert_credentials_visible(response.data)

    def test_user_cant_see_others_settings(self):
        self.client.force_authenticate(user=self.users['not_owner'])

        response = self.client.get(factories.ServiceSettingsFactory.get_url(self.settings['owned']))
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_admin_can_see_all_credentials(self):
        self.client.force_authenticate(user=self.users['staff'])

        response = self.client.get(factories.ServiceSettingsFactory.get_url(self.settings['owned']))
        self.assert_credentials_visible(response.data)

    def test_user_cant_see_shared_credentials(self):
        self.client.force_authenticate(user=self.users['owner'])

        response = self.client.get(factories.ServiceSettingsFactory.get_url(self.settings['shared']))
        self.assert_credentials_hidden(response.data)

    def assert_credentials_visible(self, data):
        for field in self.credentials:
            self.assertIn(field, data)

    def assert_credentials_hidden(self, data):
        for field in self.credentials:
            self.assertNotIn(field, data)


@ddt
class ServiceSettingsUpdateTest(test.APITransactionTestCase):
    def setUp(self):
        self.fixture = fixtures.ServiceFixture()
        self.service_settings = self.fixture.service.settings
        self.url = factories.ServiceSettingsFactory.get_url(self.service_settings)

    @data('staff', 'owner')
    def test_staff_and_owner_can_update_service_settings(self, user):
        self.assert_user_can_update_service_settings(user)

    @data('admin', 'manager')
    def test_admin_and_owner_can_not_update_service_settings(self, user):
        self.assert_user_can_not_get_service_settings(user)

    @override_waldur_core_settings(ONLY_STAFF_MANAGES_SERVICES=True)
    def test_if_only_staff_manages_services_he_can_update_it(self):
        self.assert_user_can_update_service_settings('staff')

    @data('owner', 'admin', 'manager')
    @override_waldur_core_settings(ONLY_STAFF_MANAGES_SERVICES=True)
    def test_if_only_staff_manages_services_other_users_can_not_update_it(self, user):
        self.assert_user_can_not_update_service_settings(user)

    def assert_user_can_update_service_settings(self, user):
        self.client.force_authenticate(getattr(self.fixture, user))

        response = self.update_service_settings()
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.service_settings.refresh_from_db()
        self.assertEqual(self.service_settings.name, 'Valid new name')

    def assert_user_can_not_update_service_settings(self, user):
        self.client.force_authenticate(getattr(self.fixture, user))

        response = self.update_service_settings()
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

        self.service_settings.refresh_from_db()
        self.assertNotEqual(self.service_settings.name, 'Valid new name')

    def assert_user_can_not_get_service_settings(self, user):
        self.client.force_authenticate(getattr(self.fixture, user))

        response = self.update_service_settings()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self.service_settings.refresh_from_db()
        self.assertNotEqual(self.service_settings.name, 'Valid new name')

    def update_service_settings(self):
        return self.client.patch(self.url, {'name': 'Valid new name'})


@ddt
class SharedServiceSettingUpdateTest(test.APITransactionTestCase):

    def setUp(self):
        self.fixture = fixtures.ServiceFixture()
        self.service_settings = self.fixture.service_settings
        self.service_settings.shared = True
        self.service_settings.save()
        self.url = factories.ServiceSettingsFactory.get_url(self.service_settings)

    def get_valid_payload(self):
        return {'name': 'test'}

    @data('staff', 'owner')
    def test_only_staff_and_an_owner_of_an_unshared_service_settings_can_update_the_settings(self, user):
        self.service_settings.shared = False
        self.service_settings.save()
        self.client.force_authenticate(getattr(self.fixture, user))
        payload = self.get_valid_payload()

        response = self.client.patch(self.url, data=payload)

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.service_settings.refresh_from_db()
        self.assertEqual(self.service_settings.name, payload['name'])

    @data('owner', 'manager', 'admin')
    def test_user_cannot_update_shared_service_settings_without_customer_if_he_has_no_permission(self, user):
        self.service_settings.customer = None
        self.service_settings.save()
        self.client.force_authenticate(getattr(self.fixture, user))
        payload = self.get_valid_payload()

        response = self.client.patch(self.url, data=payload)

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)
        self.service_settings.refresh_from_db()
        self.assertNotEqual(self.service_settings.name, payload['name'])

    @data('staff')
    def test_user_can_update_shared_service_settings_without_customer_if_he_has_permission(self, user):
        self.service_settings.customer = None
        self.service_settings.save()
        self.client.force_authenticate(getattr(self.fixture, user))
        payload = self.get_valid_payload()

        response = self.client.patch(self.url, data=payload)

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.service_settings.refresh_from_db()
        self.assertEqual(self.service_settings.name, payload['name'])

    @data('manager', 'admin')
    def test_user_cannot_update_shared_service_settings_with_customer_if_he_has_no_permission(self, user):
        self.client.force_authenticate(getattr(self.fixture, user))
        payload = self.get_valid_payload()

        response = self.client.patch(self.url, data=payload)

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)
        self.service_settings.refresh_from_db()
        self.assertNotEqual(self.service_settings.name, payload['name'])

    def test_user_cannot_change_unshared_settings_type(self):
        self.service_settings.shared = False
        self.service_settings.save()
        self.client.force_authenticate(user=self.fixture.owner)
        payload = {'name': 'Test backend', 'type': 2}

        response = self.client.patch(self.url, payload)

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.service_settings.refresh_from_db()
        self.assertNotEqual(self.service_settings.type, payload['type'], response.data)

    def test_user_can_change_unshared_settings_password(self):
        self.service_settings.shared = False
        self.service_settings.save()
        self.client.force_authenticate(user=self.fixture.owner)
        payload = {'password': 'secret'}

        response = self.client.patch(self.url, data=payload)

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.service_settings.refresh_from_db()
        self.assertEqual(self.service_settings.password, payload['password'], response.data)

    def test_user_cannot_change_shared_settings_password(self):
        self.client.force_authenticate(user=self.fixture.owner)
        payload = {'password': 'secret'}

        response = self.client.patch(self.url, data=payload)

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_options_are_partially_updated(self):
        required_field_name = 'availability_zone'
        self.service_settings.shared = False
        self.service_settings.options = {required_field_name: 'value'}
        self.service_settings.save()
        self.client.force_authenticate(user=self.fixture.owner)
        payload = {'tenant_name': 'secret'}

        response = self.client.patch(self.url, data=payload)

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.service_settings.refresh_from_db()
        self.assertIn(required_field_name, self.service_settings.options)


@ddt
class ServiceSettingsUpdateCertifications(test.APITransactionTestCase):

    def setUp(self):
        self.fixture = fixtures.ServiceFixture()
        self.settings = self.fixture.service_settings
        self.associated_certification = factories.ServiceCertificationFactory()
        self.settings.certifications.add(self.associated_certification)
        self.new_certification = factories.ServiceCertificationFactory()
        self.url = factories.ServiceSettingsFactory.get_url(self.settings, 'update_certifications')

    @data('staff', 'owner')
    def test_user_can_update_certifications_for_unshared_settings(self, user):
        self.client.force_authenticate(getattr(self.fixture, user))
        payload = self._get_payload(self.new_certification)

        response = self.client.post(self.url, payload)

        self.assertEqual(response.status_code, status.HTTP_200_OK, response.data)
        self.settings.refresh_from_db()
        self.assertTrue(self.settings.certifications.filter(pk=self.new_certification.pk).exists())
        self.assertFalse(self.settings.certifications.filter(pk=self.associated_certification.pk).exists())

    @data('manager', 'admin', 'global_support', 'owner')
    def test_user_can_not_update_certifications_for_shared_settings_if_he_is_not_staff(self, user):
        self.settings.shared = True
        self.settings.save()
        self.client.force_authenticate(getattr(self.fixture, user))
        payload = self._get_payload(self.associated_certification)

        response = self.client.post(self.url, payload)

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @override_waldur_core_settings(ONLY_STAFF_MANAGES_SERVICES=True)
    @data('owner', 'manager', 'admin')
    def test_if_only_staff_manages_services_other_user_can_not_update_certifications(self, user):
        self.client.force_authenticate(getattr(self.fixture, user))

        payload = self._get_payload(self.associated_certification)
        response = self.client.post(self.url, payload)

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_staff_can_update_certifications_for_shared_settings(self):
        self.settings.shared = True
        self.settings.save()
        self.client.force_authenticate(self.fixture.staff)
        payload = self._get_payload(self.new_certification)

        response = self.client.post(self.url, payload)

        self.assertEqual(response.status_code, status.HTTP_200_OK, response.data)
        self.settings.refresh_from_db()
        self.assertTrue(self.settings.certifications.filter(pk=self.new_certification.pk).exists())

    def _get_payload(self, *certifications):
        certification_urls = [{"url": factories.ServiceCertificationFactory.get_url(c)} for c in certifications]
        return {
            'certifications': certification_urls
        }



from __future__ import unicode_literals

from datetime import timedelta

from django.urls import reverse
from django.utils import timezone
from rest_framework import test, status

from waldur_core.core import utils as core_utils
from waldur_core.structure import models
from waldur_core.structure.tests import factories


class CreationTimeStatsTest(test.APITransactionTestCase):

    def setUp(self):
        # customers
        self.old_customer = factories.CustomerFactory(created=timezone.now() - timedelta(days=10))
        self.new_customer = factories.CustomerFactory(created=timezone.now() - timedelta(days=1))
        # projects
        self.old_projects = factories.ProjectFactory.create_batch(
            3, created=timezone.now() - timedelta(days=10), customer=self.old_customer)
        self.new_projects = factories.ProjectFactory.create_batch(
            3, created=timezone.now() - timedelta(days=1), customer=self.new_customer)
        # users
        self.staff = factories.UserFactory(is_staff=True)
        self.old_customer_owner = factories.UserFactory()
        self.old_customer.add_user(self.old_customer_owner, models.CustomerRole.OWNER)
        self.all_projects_admin = factories.UserFactory()
        for p in self.old_projects + self.new_projects:
            p.add_user(self.all_projects_admin, models.ProjectRole.ADMINISTRATOR)

        self.url = reverse('stats_creation_time')
        self.default_data = {
            'from': core_utils.datetime_to_timestamp(timezone.now() - timedelta(days=12)),
            'datapoints': 2,
        }

    def execute_request_with_data(self, user, extra_data):
        self.client.force_authenticate(user)
        data = self.default_data
        data.update(extra_data)
        return self.client.get(self.url, data)

    def test_staff_receive_all_stats_about_customers(self):
        # when
        response = self.execute_request_with_data(self.staff, {'type': 'customer'})
        # then
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(len(response.data), 2, 'Response has to contain 2 datapoints')
        self.assertEqual(response.data[0]['value'], 1, 'First datapoint has to contain 1 customer (old)')
        self.assertEqual(response.data[1]['value'], 1, 'Second datapoint has to contain 1 customer (new)')

    def test_customer_owner_receive_stats_only_about_his_cusotmers(self):
        # when
        response = self.execute_request_with_data(self.old_customer_owner, {'type': 'customer'})
        # then
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(len(response.data), 2, 'Response has to contain 2 datapoints')
        self.assertEqual(response.data[0]['value'], 1, 'First datapoint has to contain 1 customer (old)')
        self.assertEqual(response.data[1]['value'], 0, 'Second datapoint has to contain 0 customers')

    def test_admin_receive_info_about_his_projects(self):
        # when
        response = self.execute_request_with_data(self.all_projects_admin, {'type': 'project', 'datapoints': 3})
        # then
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(len(response.data), 3, 'Response has to contain 3 datapoints')
        self.assertEqual(response.data[0]['value'], 3, 'First datapoint has to contain 3 projects (old)')
        self.assertEqual(response.data[1]['value'], 0, 'Second datapoint has to contain 0 projects')
        self.assertEqual(response.data[2]['value'], 3, 'Third datapoint has to contain 3 projects (new)')


class BaseQuotaAggregationTest(test.APITransactionTestCase):
    def setUp(self):
        user = factories.UserFactory()
        self.client.force_authenticate(user)

        customer = factories.CustomerFactory()
        customer.add_user(user, models.CustomerRole.OWNER)

        self.project = factories.ProjectFactory(customer=customer)

    def create_links(self, limit1, usage1, limit2, usage2):
        link1 = factories.TestServiceProjectLinkFactory(project=self.project)
        link1.set_quota_limit('vcpu', limit1)
        link1.set_quota_usage('vcpu', usage1)

        link2 = factories.TestServiceProjectLinkFactory(project=self.project)
        link2.set_quota_limit('vcpu', limit2)
        link2.set_quota_usage('vcpu', usage2)


class StatsQuotaTimelineTest(BaseQuotaAggregationTest):
    def test_negative_limit(self):
        """
        If any quota limit is -1, total limit is -1
        """
        self.create_links(limit1=-1, usage1=10, limit2=2, usage2=1)

        response = self.get_response()

        self.assertEqual(-1, response.data[0]['vcpu_limit'])
        self.assertEqual(11, response.data[0]['vcpu_usage'])

    def test_positive_limit(self):
        """
        If all limits are positive they are summed up
        """
        self.create_links(limit1=10, usage1=2, limit2=100, usage2=10)

        response = self.get_response()

        self.assertEqual(110, response.data[0]['vcpu_limit'])
        self.assertEqual(12, response.data[0]['vcpu_usage'])

    def get_response(self):
        response = self.client.get(reverse('stats_quota_timeline'), data={
            'aggregate': 'project',
            'uuid': self.project.uuid.hex,
            'item': 'vcpu',
            'from': core_utils.datetime_to_timestamp(timezone.now() - timedelta(minutes=1)),
            'to': core_utils.datetime_to_timestamp(timezone.now() + timedelta(minutes=1))
        })
        return response


class StatsQuotaTest(BaseQuotaAggregationTest):

    def test_negative_limit(self):
        """
        If any quota limit is -1, total limit is -1
        """
        self.create_links(limit1=-1, usage1=10, limit2=2, usage2=1)

        response = self.get_response()

        self.assertEqual(-1, response.data['vcpu'])
        self.assertEqual(11, response.data['vcpu_usage'])

    def test_positive_limit(self):
        """
        If all limits are positive they are summed up
        """
        self.create_links(limit1=10, usage1=2, limit2=100, usage2=10)

        response = self.get_response()

        self.assertEqual(110, response.data['vcpu'])
        self.assertEqual(12, response.data['vcpu_usage'])

    def get_response(self):
        response = self.client.get(reverse('stats_quota'), data={
            'aggregate': 'project',
            'uuid': self.project.uuid.hex
        })
        return response



from ddt import ddt, data
from rest_framework import status

from waldur_core.structure.tests import factories as structure_factories

from . import factories, base_test
from .. import models


@ddt
class DefaultPriceListItemListTest(base_test.BaseCostTrackingTest):

    def setUp(self):
        super(DefaultPriceListItemListTest, self).setUp()
        self.users['regular_user'] = structure_factories.UserFactory(username='regular_user')
        self.default_price_list_item = factories.DefaultPriceListItemFactory()

    @data('staff', 'owner', 'manager', 'regular_user', 'administrator')
    def test_user_with_access_to_service_can_see_services_price_list(self, user):
        self.client.force_authenticate(self.users[user])
        data = {'page_size': models.DefaultPriceListItem.objects.count()}
        response = self.client.get(factories.DefaultPriceListItemFactory.get_list_url(), data)

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertIn(self.default_price_list_item.uuid.hex, [el['uuid'] for el in response.data])



from rest_framework import renderers

from waldur_core import __version__


class BrowsableAPIRenderer(renderers.BrowsableAPIRenderer):
    """
    HTML renderer used to self-document the API.

    This renderer populates version of the server running
    to the context.
    """

    def get_context(self, data, accepted_media_type, renderer_context):
        context = super(BrowsableAPIRenderer, self).get_context(data, accepted_media_type, renderer_context)
        context['version'] = __version__
        return context



from django.urls import NoReverseMatch
from django.utils.encoding import smart_text, force_text
from django_filters import OrderingFilter, ChoiceFilter, ModelMultipleChoiceFilter
from rest_framework import exceptions, schemas
from rest_framework.compat import coreapi
from rest_framework.fields import ChoiceField, ModelField, HiddenField
from rest_framework.permissions import AllowAny, SAFE_METHODS
from rest_framework.relations import HyperlinkedRelatedField, ManyRelatedField
from rest_framework.response import Response
from rest_framework.reverse import reverse
from rest_framework.serializers import ListSerializer, Serializer
from rest_framework.utils import formatting
from rest_framework.views import APIView
from rest_framework_swagger import renderers


from waldur_core.core import (permissions as core_permissions, views as core_views,
                              utils as core_utils, filters as core_filters, serializers as core_serializers)
from waldur_core.cost_tracking.filters import ResourceTypeFilter
from waldur_core.structure import SupportedServices, filters as structure_filters


# XXX: Drop after removing HEAD requests
class WaldurEndpointInspector(schemas.EndpointInspector):
    def get_allowed_methods(self, callback):
        """
        Return a list of the valid HTTP methods for this endpoint.
        """
        if hasattr(callback, 'actions'):
            return [method.upper() for method in callback.actions.keys() if method != 'head']

        return [
            method for method in
            callback.cls().allowed_methods if method not in ('OPTIONS', 'HEAD')
        ]


def get_entity_description(entity):
    """
    Returns description in format:
    * entity human readable name
     * docstring
    """

    try:
        entity_name = entity.__name__.strip('_')
    except AttributeError:
        # entity is a class instance
        entity_name = entity.__class__.__name__

    label = '* %s' % formatting.camelcase_to_spaces(entity_name)
    if entity.__doc__ is not None:
        entity_docstring = formatting.dedent(smart_text(entity.__doc__)).replace('\n', '\n\t')
        return '%s\n * %s' % (label, entity_docstring)

    return label


def get_validators_description(view):
    """
    Returns validators description in format:
    ### Validators:
    * validator1 name
     * validator1 docstring
    * validator2 name
     * validator2 docstring
    """
    action = getattr(view, 'action', None)
    if action is None:
        return ''

    description = ''
    validators = getattr(view, action + '_validators', [])
    for validator in validators:
        validator_description = get_entity_description(validator)
        description += '\n' + validator_description if description else validator_description

    return '### Validators:\n' + description if description else ''


def get_actions_permission_description(view, method):
    """
    Returns actions permissions description in format:
    * permission1 name
     * permission1 docstring
    * permission2 name
     * permission2 docstring
    """
    action = getattr(view, 'action', None)
    if action is None:
        return ''

    if hasattr(view, action + '_permissions'):
        permission_types = (action,)
    elif method in SAFE_METHODS:
        permission_types = ('safe_methods', '%s_extra' % action)
    else:
        permission_types = ('unsafe_methods', '%s_extra' % action)

    description = ''
    for permission_type in permission_types:
        action_perms = getattr(view, permission_type + '_permissions', [])
        for permission in action_perms:
            action_perm_description = get_entity_description(permission)
            description += '\n' + action_perm_description if description else action_perm_description

    return description


def get_permissions_description(view, method):
    """
    Returns permissions description in format:
    ### Permissions:
    * permission1 name
     * permission1 docstring
    * permission2 name
     * permission2 docstring
    """
    if not hasattr(view, 'permission_classes'):
        return ''

    description = ''
    for permission_class in view.permission_classes:
        if permission_class == core_permissions.ActionsPermission:
            actions_perm_description = get_actions_permission_description(view, method)
            if actions_perm_description:
                description += '\n' + actions_perm_description if description else actions_perm_description
            continue
        perm_description = get_entity_description(permission_class)
        description += '\n' + perm_description if description else perm_description

    return '### Permissions:\n' + description if description else ''


def get_validation_description(view, method):
    """
    Returns validation description in format:
    ### Validation:
    validate method docstring
    * field1 name
     * field1 validation docstring
    * field2 name
     * field2 validation docstring
    """
    if method not in ('PUT', 'PATCH', 'POST') or not hasattr(view, 'get_serializer'):
        return ''

    serializer = view.get_serializer()
    description = ''
    if hasattr(serializer, 'validate') and serializer.validate.__doc__ is not None:
        description += formatting.dedent(smart_text(serializer.validate.__doc__))

    for field in serializer.fields.values():
        if not hasattr(serializer, 'validate_' + field.field_name):
            continue

        field_validation = getattr(serializer, 'validate_' + field.field_name)

        if field_validation.__doc__ is not None:
            docstring = formatting.dedent(smart_text(field_validation.__doc__)).replace('\n', '\n\t')
            field_description = '* %s\n * %s' % (field.field_name, docstring)
            description += '\n' + field_description if description else field_description

    return '### Validation:\n' + description if description else ''


FIELDS = {
    'Boolean': 'boolean',
    'Char': 'string',
    'Timestamp': 'UNIX timestamp',
    'DateTime': 'DateTime',
    'URL': 'link',
    'Number': 'float',
    'UUID': 'string',
    'ContentType': 'string in form app_label.model_name',
    'Decimal': 'float',
    'Float': 'float',
    'File': 'file',
    'Email': 'email',
    'Integer': 'integer',
    'IPAddress': 'IP address',
    'HyperlinkedRelated': 'link',
}


def get_field_type(field):
    """
    Returns field type/possible values.
    """
    if isinstance(field, core_filters.MappedMultipleChoiceFilter):
        return ' | '.join(['"%s"' % f for f in sorted(field.mapped_to_model)])
    if isinstance(field, OrderingFilter) or isinstance(field, ChoiceFilter):
        return ' | '.join(['"%s"' % f[0] for f in field.extra['choices']])
    if isinstance(field, ChoiceField):
        return ' | '.join(['"%s"' % f for f in sorted(field.choices)])
    if isinstance(field, HyperlinkedRelatedField):
        if field.view_name.endswith('detail'):
            return 'link to %s' % reverse(field.view_name,
                                          kwargs={'%s' % field.lookup_field: "'%s'" % field.lookup_field})
        return reverse(field.view_name)
    if isinstance(field, structure_filters.ServiceTypeFilter):
        return ' | '.join(['"%s"' % f for f in SupportedServices.get_filter_mapping().keys()])
    if isinstance(field, ResourceTypeFilter):
        return ' | '.join(['"%s"' % f for f in SupportedServices.get_resource_models().keys()])
    if isinstance(field, core_serializers.GenericRelatedField):
        links = []
        for model in field.related_models:
            detail_view_name = core_utils.get_detail_view_name(model)
            for f in field.lookup_fields:
                try:
                    link = reverse(detail_view_name, kwargs={'%s' % f: "'%s'" % f})
                except NoReverseMatch:
                    pass
                else:
                    links.append(link)
                    break
        path = ', '.join(links)
        if path:
            return 'link to any: %s' % path
    if isinstance(field, core_filters.ContentTypeFilter):
        return "string in form 'app_label'.'model_name'"
    if isinstance(field, ModelMultipleChoiceFilter):
        return get_field_type(field.field)
    if isinstance(field, ListSerializer):
        return 'list of [%s]' % get_field_type(field.child)
    if isinstance(field, ManyRelatedField):
        return 'list of [%s]' % get_field_type(field.child_relation)
    if isinstance(field, ModelField):
        return get_field_type(field.model_field)

    name = field.__class__.__name__
    for w in ('Filter', 'Field', 'Serializer'):
        name = name.replace(w, '')
    return FIELDS.get(name, name)


def is_disabled_action(view):
    """
    Checks whether Link action is disabled.
    """
    if not isinstance(view, core_views.ActionsViewSet):
        return False

    action = getattr(view, 'action', None)
    return action in view.disabled_actions if action is not None else False


class WaldurSchemaGenerator(schemas.SchemaGenerator):
    endpoint_inspector_cls = WaldurEndpointInspector

    def create_view(self, callback, method, request=None):
        """
        Given a callback, return an actual view instance.
        """
        view = super(WaldurSchemaGenerator, self).create_view(callback, method, request)
        if is_disabled_action(view):
            view.exclude_from_schema = True

        return view

    def get_description(self, path, method, view):
        """
        Determine a link description.

        This will be based on the method docstring if one exists,
        or else the class docstring.
        """
        description = super(WaldurSchemaGenerator, self).get_description(path, method, view)

        permissions_description = get_permissions_description(view, method)
        if permissions_description:
            description += '\n\n' + permissions_description if description else permissions_description

        if isinstance(view, core_views.ActionsViewSet):
            validators_description = get_validators_description(view)
            if validators_description:
                description += '\n\n' + validators_description if description else validators_description

        validation_description = get_validation_description(view, method)
        if validation_description:
            description += '\n\n' + validation_description if description else validation_description

        return description

    def get_filter_fields(self, path, method, view):
        if not schemas.is_list_view(path, method, view):
            return []

        if not getattr(view, 'filter_backends', None):
            return []

        fields = []
        for filter_backend in view.filter_backends:
            backend = filter_backend()
            if not hasattr(backend, 'get_filter_class'):
                fields += filter_backend().get_schema_fields(view)
                continue

            filter_class = backend.get_filter_class(view, view.get_queryset())
            if not filter_class:
                continue

            for filter_name, filter_instance in filter_class().filters.items():
                filter_type = get_field_type(filter_instance)
                field = coreapi.Field(
                    name=filter_name,
                    required=False,
                    location=filter_type,
                )
                # Prevent double rendering
                if field not in fields:
                    fields.append(field)

        return fields

    def get_serializer_fields(self, path, method, view):
        """
        Return a list of `coreapi.Field` instances corresponding to any
        request body input, as determined by the serializer class.
        """
        if method not in ('PUT', 'PATCH', 'POST'):
            return []

        if not hasattr(view, 'get_serializer'):
            return []

        serializer = view.get_serializer()
        if not isinstance(serializer, Serializer):
            return []

        fields = []
        for field in serializer.fields.values():
            if field.read_only or isinstance(field, HiddenField):
                continue

            required = field.required and method != 'PATCH'
            description = force_text(field.help_text) if field.help_text else ''
            field_type = get_field_type(field)
            description += '; ' + field_type if description else field_type
            field = coreapi.Field(
                name=field.field_name,
                location='form',
                required=required,
                description=description,
                schema=schemas.field_to_schema(field),
            )
            fields.append(field)

        return fields


class WaldurSchemaView(APIView):
    permission_classes = [AllowAny]
    renderer_classes = [
        renderers.OpenAPIRenderer,
        renderers.SwaggerUIRenderer
    ]

    def get(self, request):
        generator = WaldurSchemaGenerator(
            title='Waldur MasterMind',
        )
        schema = generator.get_schema(request=request)

        if not schema:
            raise exceptions.ValidationError(
                'The schema generator did not return a schema Document'
            )

        return Response(schema)



from django.contrib.auth import get_user_model
from rest_framework import status, test

User = get_user_model()


class DocsRenderTest(test.APITransactionTestCase):

    def test_swagger_docs_render(self):
        user, _ = User.objects.get_or_create(username='waldur_docs_tester', is_staff=True)
        self.client.force_authenticate(user=user)

        response = self.client.get('/docs/')
        self.assertEqual(response.status_code, status.HTTP_200_OK)



import factory
import factory.fuzzy

from rest_framework.reverse import reverse

from waldur_core.structure import models as structure_models
from waldur_core.structure.tests import factories as structure_factories
from waldur_core.users import models


class InvitationBaseFactory(factory.DjangoModelFactory):
    @classmethod
    def get_list_url(cls):
        return 'http://testserver' + reverse('user-invitation-list')

    @classmethod
    def get_url(cls, invitation, action=None):
        url = 'http://testserver' + reverse('user-invitation-detail', kwargs={'uuid': invitation.uuid})
        return url if action is None else url + action + '/'


class ProjectInvitationFactory(InvitationBaseFactory):
    class Meta(object):
        model = models.Invitation

    customer = factory.SelfAttribute('project.customer')
    project = factory.SubFactory(structure_factories.ProjectFactory)
    project_role = structure_models.ProjectRole.MANAGER
    link_template = factory.Sequence(lambda n: 'http://testinvitation%1.com/project/{uuid}' % n)
    email = factory.Sequence(lambda n: 'test%s@invitation.com' % n)

    @classmethod
    def get_url(cls, invitation=None, action=None):
        if invitation is None:
            invitation = ProjectInvitationFactory()
        return super(ProjectInvitationFactory, cls).get_url(invitation, action)


class CustomerInvitationFactory(InvitationBaseFactory):
    class Meta(object):
        model = models.Invitation

    customer = factory.SubFactory(structure_factories.CustomerFactory)
    customer_role = structure_models.CustomerRole.OWNER
    link_template = factory.Sequence(lambda n: 'http://testinvitation%1.com/customer/{uuid}' % n)
    email = factory.Sequence(lambda n: 'test%s@invitation.com' % n)

    @classmethod
    def get_url(cls, invitation=None, action=None):
        if invitation is None:
            invitation = CustomerInvitationFactory()
        return super(CustomerInvitationFactory, cls).get_url(invitation, action)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import collections

from django.conf import settings
from django.contrib.auth import get_user_model
from django.db import migrations, models
import model_utils.fields
import waldur_core.core.fields
import waldur_core.structure.models
import django.utils.timezone


def migrate_permissions(apps, schema_editor):
    customer_owners, project_managers, project_admins = fetch_users()
    migrate_customer_permissions(apps, customer_owners)
    migrate_project_permissions(apps, project_managers, project_admins)


def fetch_users():
    UserGroup = get_user_model().groups.through

    customer_owners = collections.defaultdict(list)
    project_managers = collections.defaultdict(list)
    project_admins = collections.defaultdict(list)
    arrays = {
        'owner': customer_owners,
        'mgr': project_managers,
        'admin': project_admins,
    }
    user_groups = UserGroup.objects.filter(group__name__startswith='Role:')\
                           .exclude(user=None).select_related('group', 'user')
    for user_group in user_groups:
        parts = user_group.group.name.split(' ')
        if len(parts) != 3:
            continue
        _, uuid, role = parts
        if role in arrays:
            arrays[role][uuid].append(user_group.user)
    return customer_owners, project_managers, project_admins


def migrate_customer_permissions(apps, customer_owners):
    Customer = apps.get_model('structure', 'Customer')
    CustomerPermission = apps.get_model('structure', 'CustomerPermission')

    customers = Customer.objects.filter(uuid__in=customer_owners.keys())
    customers_map = {customer.uuid.hex: customer for customer in customers}

    customer_permissions = [
        CustomerPermission(customer=customer, user_id=user.id, role='owner')
        for uuid, customer in customers_map.items()
        for user in customer_owners[uuid]
    ]
    CustomerPermission.objects.bulk_create(customer_permissions)


def migrate_project_permissions(apps, project_managers, project_admins):
    Project = apps.get_model('structure', 'Project')
    ProjectPermission = apps.get_model('structure', 'ProjectPermission')

    projects = Project.objects.filter(uuid__in=project_managers.keys() + project_admins.keys())
    projects_map = {project.uuid.hex: project for project in projects}

    project_permissions = [
        ProjectPermission(project=project, user_id=user.id, role='manager')
        for uuid, project in projects_map.items()
        for user in project_managers[uuid]
    ] + [
        ProjectPermission(project=project, user_id=user.id, role='admin')
        for uuid, project in projects_map.items()
        for user in project_admins[uuid]
    ]
    ProjectPermission.objects.bulk_create(project_permissions)


class Migration(migrations.Migration):

    dependencies = [
        ('structure', '0037_remove_customer_billing_backend_id'),
    ]

    operations = [
        migrations.CreateModel(
            name='CustomerPermission',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False)),
                ('expiration_time', models.DateTimeField(null=True, blank=True)),
                ('is_active', models.BooleanField(default=True, db_index=True)),
                ('role', waldur_core.structure.models.CustomerRole(db_index=True, max_length=30, choices=[('owner', 'Owner')])),
                ('created_by', models.ForeignKey(related_name='+', blank=True, to=settings.AUTH_USER_MODEL, null=True)),
                ('customer', models.ForeignKey(verbose_name='organization', related_name='permissions', to='structure.Customer')),
                ('user', models.ForeignKey(to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='ProjectPermission',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('uuid', waldur_core.core.fields.UUIDField()),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False)),
                ('expiration_time', models.DateTimeField(null=True, blank=True)),
                ('is_active', models.BooleanField(default=True, db_index=True)),
                ('role', waldur_core.structure.models.ProjectRole(db_index=True, max_length=30, choices=[('admin', 'Administrator'), ('manager', 'Manager')])),
                ('created_by', models.ForeignKey(related_name='+', blank=True, to=settings.AUTH_USER_MODEL, null=True)),
                ('project', models.ForeignKey(related_name='permissions', to='structure.Project')),
                ('user', models.ForeignKey(to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.AlterUniqueTogether(
            name='projectpermission',
            unique_together=set([('project', 'role', 'user', 'is_active')]),
        ),
        migrations.AlterUniqueTogether(
            name='customerpermission',
            unique_together=set([('customer', 'role', 'user', 'is_active')]),
        ),
        migrations.RunPython(migrate_permissions)
    ]



from ddt import ddt, data
from django.test import TestCase
from mock import patch, Mock

from waldur_core.core import utils
from waldur_core.structure import tasks
from waldur_core.structure.tests import factories, models


class TestDetectVMCoordinatesTask(TestCase):

    @patch('requests.get')
    def test_task_sets_coordinates(self, mock_request_get):
        ip_address = "127.0.0.1"
        expected_latitude = 20
        expected_longitude = 20
        instance = factories.TestNewInstanceFactory()

        mock_request_get.return_value.ok = True
        response = {"ip": ip_address, "latitude": expected_latitude, "longitude": expected_longitude}
        mock_request_get.return_value.json.return_value = response
        tasks.detect_vm_coordinates(utils.serialize_instance(instance))

        instance.refresh_from_db()
        self.assertEqual(instance.latitude, expected_latitude)
        self.assertEqual(instance.longitude, expected_longitude)

    @patch('requests.get')
    def test_task_does_not_set_coordinates_if_response_is_not_ok(self, mock_request_get):
        ip_address = "127.0.0.1"
        instance = factories.TestNewInstanceFactory()

        mock_request_get.return_value.ok = False
        tasks.detect_vm_coordinates(utils.serialize_instance(instance))

        instance.refresh_from_db()
        self.assertIsNone(instance.latitude)
        self.assertIsNone(instance.longitude)


@ddt
class ThrottleProvisionTaskTest(TestCase):

    @data(
        dict(size=tasks.ThrottleProvisionTask.DEFAULT_LIMIT + 1, retried=True),
        dict(size=tasks.ThrottleProvisionTask.DEFAULT_LIMIT - 1, retried=False),
    )
    def test_if_limit_is_reached_provisioning_is_delayed(self, params):
        link = factories.TestServiceProjectLinkFactory()
        factories.TestNewInstanceFactory.create_batch(
            size=params['size'],
            state=models.TestNewInstance.States.CREATING,
            service_project_link=link)
        vm = factories.TestNewInstanceFactory(
            state=models.TestNewInstance.States.CREATION_SCHEDULED,
            service_project_link=link)
        serialized_vm = utils.serialize_instance(vm)
        mocked_retry = Mock()
        tasks.ThrottleProvisionTask.retry = mocked_retry
        tasks.ThrottleProvisionTask().si(
            serialized_vm,
            'create',
            state_transition='begin_starting').apply()
        self.assertEqual(mocked_retry.called, params['retried'])



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import waldur_core.core.fields


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Quota',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('uuid', waldur_core.core.fields.UUIDField()),
                ('name', models.CharField(max_length=150, verbose_name='name')),
                ('limit', models.FloatField(default=-1)),
                ('usage', models.FloatField(default=0)),
                ('object_id', models.PositiveIntegerField()),
                ('content_type', models.ForeignKey(to='contenttypes.ContentType')),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.AlterUniqueTogether(
            name='quota',
            unique_together=set([('name', 'content_type', 'object_id')]),
        ),
    ]



from __future__ import unicode_literals

import uuid

import django_filters
from django.db.models import Q
from django_filters.rest_framework import DjangoFilterBackend

from waldur_core.core import filters as core_filters
from waldur_core.users import models


class InvitationFilter(django_filters.FilterSet):
    project = django_filters.UUIDFilter(
        name='project__uuid',
    )
    project_url = core_filters.URLFilter(
        view_name='project-detail',
        name='project__uuid',
    )
    state = django_filters.MultipleChoiceFilter(choices=models.Invitation.State.CHOICES)

    o = django_filters.OrderingFilter(fields=('email', 'state', 'created'))

    class Meta(object):
        model = models.Invitation
        fields = [
            'email',
            'civil_number',
            'customer_role',
            'project_role',
        ]


class InvitationCustomerFilterBackend(DjangoFilterBackend):
    url_filter = core_filters.URLFilter(
        view_name='customer-detail',
        name='customer__uuid',
    )

    def filter_queryset(self, request, queryset, view):
        customer_uuid = self.extract_customer_uuid(request)
        if not customer_uuid:
            return queryset

        try:
            uuid.UUID(customer_uuid)
        except ValueError:
            return queryset.none()

        query = Q(customer__uuid=customer_uuid)
        query |= Q(project__customer__uuid=customer_uuid)
        return queryset.filter(query)

    def extract_customer_uuid(self, request):
        if 'customer_url' in request.query_params:
            return self.url_filter.get_uuid(request.query_params['customer_url'])

        if 'customer' in request.query_params:
            return request.query_params['customer']



import factory
from factory import fuzzy

from django.urls import reverse
from django.contrib.contenttypes.models import ContentType

from waldur_core.cost_tracking import models, CostTrackingStrategy, ConsumableItem
from waldur_core.structure.tests import models as test_models
from waldur_core.structure.tests import factories as structure_factories


class PriceEstimateFactory(factory.DjangoModelFactory):
    class Meta(object):
        model = models.PriceEstimate

    scope = factory.SubFactory(structure_factories.ProjectFactory)
    total = factory.Iterator([10, 100, 1000, 10000, 980, 42])
    month = factory.Iterator(range(1, 13))
    year = factory.Iterator(range(2012, 2016))

    @classmethod
    def get_list_url(self, action=None):
        url = 'http://testserver' + reverse('priceestimate-list')
        return url if action is None else url + action + '/'

    @classmethod
    def get_url(self, price_estimate, action=None):
        if price_estimate is None:
            price_estimate = PriceEstimateFactory()
        url = 'http://testserver' + reverse('priceestimate-detail', kwargs={'uuid': price_estimate.uuid})
        return url if action is None else url + action + '/'


class ConsumptionDetailsFactory(factory.DjangoModelFactory):
    class Meta:
        model = models.ConsumptionDetails

    price_estimate = factory.SubFactory(PriceEstimateFactory)


class AbstractPriceListItemFactory(factory.DjangoModelFactory):
    class Meta(object):
        model = models.AbstractPriceListItem
        abstract = True

    value = factory.Iterator([10, 100, 1000, 10000, 1313, 13])
    units = factory.Iterator(['USD', 'EUR', 'UAH', 'OMR'])


class DefaultPriceListItemFactory(AbstractPriceListItemFactory):
    class Meta(object):
        model = models.DefaultPriceListItem

    resource_content_type = factory.LazyAttribute(
        lambda _: ContentType.objects.get_for_model(test_models.TestNewInstance))

    key = factory.Sequence(lambda n: 'price list item %s' % n)
    item_type = factory.Iterator(['flavor', 'storage'])

    @classmethod
    def get_list_url(self):
        return 'http://testserver' + reverse('defaultpricelistitem-list')

    @classmethod
    def get_url(self, default_price_list_item, action=None):
        if default_price_list_item is None:
            default_price_list_item = DefaultPriceListItemFactory()
        url = 'http://testserver' + reverse(
            'defaultpricelistitem-detail', kwargs={'uuid': default_price_list_item.uuid})
        return url if action is None else url + action + '/'


class PriceListItemFactory(AbstractPriceListItemFactory):
    class Meta(object):
        model = models.PriceListItem

    service = factory.SubFactory(structure_factories.TestServiceFactory)
    default_price_list_item = factory.SubFactory(DefaultPriceListItemFactory)

    @classmethod
    def get_list_url(self):
        return 'http://testserver' + reverse('pricelistitem-list')

    @classmethod
    def get_url(self, price_list_item, action=None):
        if price_list_item is None:
            price_list_item = PriceListItemFactory()
        url = 'http://testserver' + reverse('pricelistitem-detail', kwargs={'uuid': price_list_item.uuid})
        return url if action is None else url + action + '/'


class TestNewInstanceCostTrackingStrategy(CostTrackingStrategy):
    resource_class = test_models.TestNewInstance

    class Types(object):
        STORAGE = 'storage'
        RAM = 'ram'
        CORES = 'cores'
        QUOTAS = 'quotas'
        FLAVOR = 'flavor'

    @classmethod
    def get_configuration(cls, resource):
        States = test_models.TestNewInstance.States
        if resource.state == States.ERRED:
            return {}
        resource_quota_usage = resource.quotas.get(name=test_models.TestNewInstance.Quotas.test_quota).usage
        consumables = {
            ConsumableItem(item_type=cls.Types.STORAGE, key='1 MB'): resource.disk,
            ConsumableItem(item_type=cls.Types.QUOTAS, key='test_quota'): resource_quota_usage,
        }
        if resource.runtime_state == 'online':
            consumables.update({
                ConsumableItem(item_type=cls.Types.RAM, key='1 MB'): resource.ram,
                ConsumableItem(item_type=cls.Types.CORES, key='1 core'): resource.cores,
            })
        if resource.flavor_name:
            consumables[ConsumableItem(item_type=cls.Types.FLAVOR, key=resource.flavor_name)] = 1
        return consumables

    @classmethod
    def get_consumable_items(cls):
        return [
            ConsumableItem(cls.Types.STORAGE, "1 MB", units='MB', name='Storage'),
            ConsumableItem(cls.Types.RAM, "1 MB", units='MB', name='RAM', default_price=1),
            ConsumableItem(cls.Types.CORES, "1 core", name='Cores'),
            ConsumableItem(cls.Types.QUOTAS, "test_quota", name='Test quota'),
            ConsumableItem(cls.Types.FLAVOR, "small", name='Small flavor'),
            ConsumableItem(cls.Types.FLAVOR, "medium", name='Medium flavor'),
            ConsumableItem(cls.Types.FLAVOR, "large", name='Large flavor'),
        ]



import mock
from django.test import TestCase

from waldur_core.logging import models as logging_models
from waldur_core.logging.tests import factories as logging_factories
from waldur_core.structure.filters import AggregateFilter
from waldur_core.structure.tests import factories


class AggregateFilterTest(TestCase):

    def setUp(self):
        self.customer = factories.CustomerFactory()
        self.project = factories.ProjectFactory()
        self.aggregate_filter = AggregateFilter()
        self.queryset = logging_models.Alert.objects

    def test_service_alert_is_returned_when_aggregate_customer_is_the_same_as_its_scope_customer(self):
        scope = factories.TestServiceFactory(customer=self.customer)
        alert = logging_factories.AlertFactory(scope=scope)

        result = self._make_aggregate_request('customer', self.customer.uuid.hex)

        self.assertEqual(len(result), 1)
        self.assertTrue(result.filter(uuid=alert.uuid).exists())

    def test_project_alert_is_not_returned_when_its_scope_belongs_to_another_customer(self):
        alert = logging_factories.AlertFactory(scope=factories.ProjectFactory())

        result = self._make_aggregate_request('customer', self.customer.uuid.hex)

        self.assertFalse(result.filter(uuid=alert.uuid).exists())

    def test_only_alerts_where_scopes_customer_is_the_aggregated_one_are_returned(self):
        customer_related_alerts = []
        logging_factories.AlertFactory(scope=factories.ProjectFactory())
        spl = factories.TestServiceProjectLinkFactory(service__customer=self.customer)
        customer_related_alerts.append(logging_factories.AlertFactory(scope=spl))
        service = factories.TestServiceFactory(customer=self.customer)
        customer_related_alerts.append(logging_factories.AlertFactory(scope=service))
        expected_alerts_ids = [alert.uuid for alert in customer_related_alerts]

        result = self._make_aggregate_request('customer', self.customer.uuid.hex)
        actual_alerts_ids = [alert.uuid for alert in result]

        self.assertItemsEqual(expected_alerts_ids, actual_alerts_ids)

    def test_service_project_link_alert_is_not_returned_when_its_scope_is_related_to_another_project(self):
        not_owned_alert = logging_factories.AlertFactory(scope=factories.TestServiceProjectLinkFactory())
        spl = factories.TestServiceProjectLinkFactory(project=self.project)
        owned_alert = logging_factories.AlertFactory(scope=spl)

        result = self._make_aggregate_request('project', self.project.uuid.hex)

        self.assertTrue(result.filter(uuid=owned_alert.uuid).exists())
        self.assertFalse(result.filter(uuid=not_owned_alert.uuid).exists())

    def _make_aggregate_request(self, aggregate_by, uuid):
        request = mock.Mock()
        request.query_params = {
            'aggregate': aggregate_by,
            'uuid': uuid,
        }

        return self.aggregate_filter.filter(request, self.queryset, None)



from django.core.management.base import BaseCommand

from waldur_core.structure.models import ServiceProjectLink


class Command(BaseCommand):
    help = """ Cleanup duplicate service project links """

    def handle(self, *args, **options):
        deleted_count = 0
        for model in ServiceProjectLink.get_all_models():
            seen = set()
            for obj in model.objects.all():
                key = (obj.service.pk, obj.project.pk)
                if key in seen:
                    obj.delete()
                    deleted_count += 1
                else:
                    seen.add(key)

        if deleted_count == 0:
            self.stdout.write('Duplicate service project links not found')
        else:
            self.stdout.write('%s duplicate service project links deleted' % deleted_count)



from django.contrib.contenttypes import models as ct_models

from waldur_core.logging import models


def remove_related_alerts(sender, instance, **kwargs):
    content_type = ct_models.ContentType.objects.get_for_model(instance)
    for alert in models.Alert.objects.filter(
            object_id=instance.id, content_type=content_type, closed__isnull=True).iterator():
        alert.close()



from django.urls import reverse
import factory
import factory.fuzzy

from waldur_core.quotas import models


class QuotaFactory(factory.DjangoModelFactory):
    class Meta:
        model = models.Quota

    limit = factory.fuzzy.FuzzyFloat(low=16.0, high=1024.0)
    usage = factory.LazyAttribute(lambda q: q.limit / 2)
    name = factory.Iterator(['vcpu', 'storage', 'max_instances', 'ram'])

    @classmethod
    def get_list_url(self):
        return 'http://testserver' + reverse('quota-list')

    @classmethod
    def get_url(self, quota, action=None):
        if quota is None:
            quota = QuotaFactory()
        url = 'http://testserver' + reverse('quota-detail', kwargs={'uuid': quota.uuid})
        return url if action is None else url + action + '/'



from ddt import data, ddt

from django.core.exceptions import ValidationError
from rest_framework import test

from waldur_core.core import validators
from waldur_core.structure.models import Customer


class NameValidationTest(test.APITransactionTestCase):
    def test_name_should_have_at_least_one_non_whitespace_character(self):
        with self.assertRaises(ValidationError):
            customer = Customer(name='      ')
            customer.full_clean()


@ddt
class MinCronValueValidatorTest(test.APITransactionTestCase):

    @data('*/1 * * * *', '*/10 * * * *', '*/59 * * * *')
    def test_validator_raises_validation_error_if_given_schedule_value_is_less_than_1_hours(self, value):
        validator = validators.MinCronValueValidator(limit_value=1)
        with self.assertRaises(ValidationError):
            validator(value)

    @data('hello world', '* * * * * *', '*/59')
    def test_validator_raises_validation_error_if_given_format_is_not_valid(self, value):
        validator = validators.MinCronValueValidator(limit_value=1)
        with self.assertRaises(ValidationError):
            validator(value)

    @data('0 * * * *', '0 0 * * *', '0 0 0 * *', '0 0 * * 0', '0 0 1 * *', '0 0 1 1 *', '0 0 1 1 *')
    def test_validator_does_not_raise_error_if_schedule_is_greater_than_or_equal_1_hour(self, value):
        validator = validators.MinCronValueValidator(limit_value=1)
        validator(value)


class CIDRListValidatorTest(test.APITransactionTestCase):

    def test_validator_accepts_valid_cidr_list(self):
        validators.validate_cidr_list('fc00::/7, 127.0.0.1/32')

    def test_validator_accepts_empty_list(self):
        validators.validate_cidr_list('   ')

    def test_invalid_values_are_rejected(self):
        with self.assertRaises(ValidationError):
            validators.validate_cidr_list('hello/25')

    def test_space_separated_list_rejected(self):
        with self.assertRaises(ValidationError):
            validators.validate_cidr_list('fc00::/7  127.0.0.1/32')



from celery import shared_task

from waldur_core.core import utils as core_utils


@shared_task
def pull_instance(serialized_instance, pulled_disk):
    """ Test-only task that allows to emulate pull operation """
    instance = core_utils.deserialize_instance(serialized_instance)
    instance.disk = pulled_disk
    instance.save()



import calendar
import datetime
import importlib
import re

import os
import time

from collections import OrderedDict
from operator import itemgetter

from django.apps import apps
from django.contrib.auth import get_user_model
from django.core.management import call_command
from django.http import QueryDict
from django.urls import resolve
from django.utils import timezone
from django.utils.crypto import get_random_string
from django.utils.encoding import force_text


def sort_dict(unsorted_dict):
    """
    Return a OrderedDict ordered by key names from the :unsorted_dict:
    """
    sorted_dict = OrderedDict()
    # sort items before inserting them into a dict
    for key, value in sorted(unsorted_dict.items(), key=itemgetter(0)):
        sorted_dict[key] = value
    return sorted_dict


def format_time_and_value_to_segment_list(time_and_value_list, segments_count, start_timestamp,
                                          end_timestamp, average=False):
    """
    Format time_and_value_list to time segments

    Parameters
    ^^^^^^^^^^
    time_and_value_list: list of tuples
        Have to be sorted by time
        Example: [(time, value), (time, value) ...]
    segments_count: integer
        How many segments will be in result
    Returns
    ^^^^^^^
    List of dictionaries
        Example:
        [{'from': time1, 'to': time2, 'value': sum_of_values_from_time1_to_time2}, ...]
    """
    segment_list = []
    time_step = (end_timestamp - start_timestamp) / segments_count
    for i in range(segments_count):
        segment_start_timestamp = start_timestamp + time_step * i
        segment_end_timestamp = segment_start_timestamp + time_step
        value_list = [
            value for time, value in time_and_value_list
            if time >= segment_start_timestamp and time < segment_end_timestamp]
        segment_value = sum(value_list)
        if average and len(value_list) != 0:
            segment_value /= len(value_list)

        segment_list.append({
            'from': segment_start_timestamp,
            'to': segment_end_timestamp,
            'value': segment_value,
        })
    return segment_list


def datetime_to_timestamp(datetime):
    return int(time.mktime(datetime.timetuple()))


def timestamp_to_datetime(timestamp, replace_tz=True):
    dt = datetime.datetime.fromtimestamp(int(timestamp))
    if replace_tz:
        dt = dt.replace(tzinfo=timezone.get_current_timezone())
    return dt


def timeshift(**kwargs):
    return timezone.now().replace(microsecond=0) + datetime.timedelta(**kwargs)


def hours_in_month(month=None, year=None):
    now = datetime.datetime.now()
    if not month:
        month = now.month
    if not year:
        year = now.year

    days_in_month = calendar.monthrange(year, month)[1]
    return 24 * days_in_month


def month_start(date):
    return timezone.make_aware(datetime.datetime(day=1, month=date.month, year=date.year))


def month_end(date):
    days_in_month = calendar.monthrange(date.year, date.month)[1]
    last_day_of_month = datetime.date(month=date.month, year=date.year, day=days_in_month)
    last_second_of_month = datetime.datetime.combine(last_day_of_month, datetime.time.max)
    return timezone.make_aware(last_second_of_month, timezone.get_current_timezone())


def pwgen(pw_len=16):
    """ Generate a random password with the given length.
        Allowed chars does not have "I" or "O" or letters and
        digits that look similar -- just to avoid confusion.
    """
    return get_random_string(pw_len, 'abcdefghjkmnpqrstuvwxyz'
                                     'ABCDEFGHJKLMNPQRSTUVWXYZ'
                                     '23456789')


def serialize_instance(instance):
    """ Serialize Django model instance """
    model_name = force_text(instance._meta)
    return '{}:{}'.format(model_name, instance.pk)


def deserialize_instance(serialized_instance):
    """ Deserialize Django model instance """
    model_name, pk = serialized_instance.split(':')
    model = apps.get_model(model_name)
    return model._default_manager.get(pk=pk)


def serialize_class(cls):
    """ Serialize Python class """
    return '{}:{}'.format(cls.__module__, cls.__name__)


def deserialize_class(serilalized_cls):
    """ Deserialize Python class """
    module_name, cls_name = serilalized_cls.split(':')
    module = importlib.import_module(module_name)
    return getattr(module, cls_name)


def clear_url(url):
    """ Remove domain and protocol from url """
    if url.startswith('http'):
        return '/' + url.split('/', 3)[-1]
    return url


def get_model_from_resolve_match(match):
    queryset = match.func.cls.queryset
    if queryset is not None:
        return queryset.model
    else:
        return match.func.cls.model


def instance_from_url(url, user=None):
    """ Restore instance from URL """
    # XXX: This circular dependency will be removed then filter_queryset_for_user
    # will be moved to model manager method
    from waldur_core.structure.managers import filter_queryset_for_user

    url = clear_url(url)
    match = resolve(url)
    model = get_model_from_resolve_match(match)
    queryset = model.objects.all()
    if user is not None:
        queryset = filter_queryset_for_user(model.objects.all(), user)
    return queryset.get(**match.kwargs)


def get_detail_view_name(model):
    if model is NotImplemented:
        raise AttributeError('Cannot get detail view name for not implemented model')

    if hasattr(model, 'get_url_name') and callable(model.get_url_name):
        return '%s-detail' % model.get_url_name()

    return '%s-detail' % model.__name__.lower()


def get_list_view_name(model):
    if model is NotImplemented:
        raise AttributeError('Cannot get list view name for not implemented model')

    if hasattr(model, 'get_url_name') and callable(model.get_url_name):
        return '%s-list' % model.get_url_name()

    return '%s-list' % model.__name__.lower()


def get_fake_context():
    user = get_user_model()()
    request = type('R', (object,), {'method': 'GET', 'user': user, 'query_params': QueryDict()})
    return {'request': request, 'user': user}


def camel_case_to_underscore(name):
    s1 = re.sub('(.)([A-Z][a-z]+)', r'\1_\2', name)
    return re.sub('([a-z0-9])([A-Z])', r'\1_\2', s1).lower()


def silent_call(name, *args, **options):
    call_command(name, stdout=open(os.devnull, 'w'), *args, **options)



from __future__ import unicode_literals

from croniter import croniter
from django.core.exceptions import ValidationError
from django.core.validators import BaseValidator, URLValidator
from django.utils import timezone
from django.utils.deconstruct import deconstructible
from django.utils.translation import ugettext_lazy as _
from iptools.ipv4 import validate_cidr as is_valid_ipv4_cidr
from iptools.ipv6 import validate_cidr as is_valid_ipv6_cidr

from waldur_core.core import exceptions


def validate_cron_schedule(value):
    try:
        base_time = timezone.now()
        croniter(value, base_time)
    except (KeyError, ValueError) as e:
        raise ValidationError(str(e))


@deconstructible
class MinCronValueValidator(BaseValidator):
    """
    Validate that the period of cron schedule is greater than or equal to provided limit_value in hours, 
    otherwise raise ValidationError.
    """
    message = _('Ensure schedule period is greater than or equal to %(limit_value)s hour(s).')
    code = 'min_cron_value'

    def compare(self, cleaned, limit_value):
        validate_cron_schedule(cleaned)

        now = timezone.now()
        schedule = croniter(cleaned, now)
        closest_schedule = schedule.get_next(timezone.datetime)
        next_schedule = schedule.get_next(timezone.datetime)
        schedule_interval_in_hours = (next_schedule - closest_schedule).total_seconds() / 3600
        return schedule_interval_in_hours < limit_value


def validate_name(value):
    if len(value.strip()) == 0:
        raise ValidationError(_('Ensure that name has at least one non-whitespace character.'))


class StateValidator(object):

    def __init__(self, *valid_states):
        self.valid_states = valid_states

    def __call__(self, resource):
        if resource.state not in self.valid_states:
            states_names = dict(resource.States.CHOICES)
            valid_states_names = [str(states_names[state]) for state in self.valid_states]
            raise exceptions.IncorrectStateException(_('Valid states for operation: %s.') % ', '.join(valid_states_names))


class RuntimeStateValidator(StateValidator):

    def __call__(self, resource):
        if resource.runtime_state not in self.valid_states:
            raise exceptions.IncorrectStateException(_('Valid runtime states for operation: %s.') % ', '.join(self.valid_states))


class BackendURLValidator(URLValidator):
    schemes = ['ldap', 'ldaps', 'http', 'https']


def is_valid_ipv46_cidr(value):
    return is_valid_ipv6_cidr(value) or is_valid_ipv4_cidr(value)


def validate_cidr_list(value):
    if not value.strip():
        return
    invalid_items = []
    for item in value.split(','):
        item = item.strip()
        if not is_valid_ipv46_cidr(item):
            invalid_items.append(item)
    if invalid_items:
        raise ValidationError(
            message=_('The following items are invalid: %s'),
            code='invalid',
            params=', '.join(invalid_items),
        )



import datetime


def get_period(request):
    period = request.query_params.get('period')
    return period or format_period(datetime.date.today())


def format_period(date):
    return '%d-%02d' % (date.year, date.month)


def to_list(xs):
    return xs if isinstance(xs, list) else [xs]



from __future__ import unicode_literals

import threading

from django.utils.deprecation import MiddlewareMixin


_locals = threading.local()


def get_event_context():
    return getattr(_locals, 'context', None)


def set_event_context(context):
    _locals.context = context


def reset_event_context():
    if hasattr(_locals, 'context'):
        del _locals.context


def set_current_user(user):
    context = get_event_context() or {}
    context.update(user._get_log_context('user'))
    set_event_context(context)


def get_ip_address(request):
    """
    Correct IP address is expected as first element of HTTP_X_FORWARDED_FOR or REMOTE_ADDR 
    """
    if 'HTTP_X_FORWARDED_FOR' in request.META:
        return request.META['HTTP_X_FORWARDED_FOR'].split(',')[0].strip()
    else:
        return request.META['REMOTE_ADDR']


class CaptureEventContextMiddleware(MiddlewareMixin):
    def process_request(self, request):
        context = {'ip_address': get_ip_address(request)}

        user = getattr(request, 'user', None)
        if user and not user.is_anonymous:
            context.update(user._get_log_context('user'))

        set_event_context(context)

    def process_response(self, request, response):
        reset_event_context()
        return response



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2017-10-20 08:47
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0007_user_token_lifetime'),
    ]

    operations = [
        migrations.AddField(
            model_name='sshpublickey',
            name='is_shared',
            field=models.BooleanField(default=False),
        ),
    ]



# encoding: utf-8
from __future__ import unicode_literals

import django.contrib.auth

import factory
import factory.fuzzy

from rest_framework.reverse import reverse

from waldur_core.core import models as core_models
from waldur_core.structure import models

from . import TestConfig
from . import models as test_models


class UserFactory(factory.DjangoModelFactory):
    class Meta(object):
        model = django.contrib.auth.get_user_model()

    username = factory.Sequence(lambda n: 'john%s' % n)
    civil_number = factory.Sequence(lambda n: '%08d' % n)
    email = factory.LazyAttribute(lambda o: '%s@example.org' % o.username)
    full_name = factory.Sequence(lambda n: 'John Doe%s' % n)
    native_name = factory.Sequence(lambda n: 'Jöhn Dõe%s' % n)
    organization = factory.Sequence(lambda n: 'Organization %s' % n)
    phone_number = factory.Sequence(lambda n: '555-555-%s-2' % n)
    description = factory.Sequence(lambda n: 'Description %s' % n)
    job_title = factory.Sequence(lambda n: 'Job %s' % n)
    is_staff = False
    is_active = True

    @factory.post_generation
    def customers(self, create, extracted, **kwargs):
        if not create:
            return

        if extracted:
            for customer in extracted:
                self.customers.add(customer)

    @classmethod
    def get_url(cls, user=None, action=None):
        if user is None:
            user = UserFactory()
        url = 'http://testserver' + reverse('user-detail', kwargs={'uuid': user.uuid})
        return url if action is None else url + action + '/'

    @classmethod
    def get_password_url(self, user):
        return 'http://testserver' + reverse('user-detail', kwargs={'uuid': user.uuid}) + 'password/'

    @classmethod
    def get_list_url(self):
        return 'http://testserver' + reverse('user-list')


class SshPublicKeyFactory(factory.DjangoModelFactory):
    class Meta(object):
        model = core_models.SshPublicKey

    user = factory.SubFactory(UserFactory)
    name = factory.Sequence(lambda n: 'ssh_public_key%s' % n)
    public_key = factory.Sequence(lambda n:
        "ssh-rsa AAAAB3NzaC1yc2EAAAADAQABAAABAQDDURXDP5YhOQUYoDuTxJ84DuzqMJYJqJ8+SZT28"
        "TtLm5yBDRLKAERqtlbH2gkrQ3US58gd2r8H9jAmQOydfvgwauxuJUE4eDpaMWupqquMYsYLB5f+vVGhdZbbzfc6DTQ2rY"
        "dknWoMoArlG7MvRMA/xQ0ye1muTv+mYMipnd7Z+WH0uVArYI9QBpqC/gpZRRIouQ4VIQIVWGoT6M4Kat5ZBXEa9yP+9du"
        "D2C05GX3gumoSAVyAcDHn/xgej9pYRXGha4l+LKkFdGwAoXdV1z79EG1+9ns7wXuqMJFHM2KDpxAizV0GkZcojISvDwuh"
        "vEAFdOJcqjyyH4%010d test" % n
    )

    @classmethod
    def get_url(self, key):
        if key is None:
            key = SshPublicKeyFactory()
        return 'http://testserver' + reverse('sshpublickey-detail', kwargs={'uuid': str(key.uuid)})

    @classmethod
    def get_list_url(self):
        return 'http://testserver' + reverse('sshpublickey-list')


class CustomerFactory(factory.DjangoModelFactory):
    class Meta(object):
        model = models.Customer

    name = factory.Sequence(lambda n: 'Customer%s' % n)
    abbreviation = factory.Sequence(lambda n: 'Cust%s' % n)
    contact_details = factory.Sequence(lambda n: 'contacts %s' % n)

    @classmethod
    def get_url(cls, customer=None, action=None):
        if customer is None:
            customer = CustomerFactory()
        url = 'http://testserver' + reverse('customer-detail', kwargs={'uuid': customer.uuid})
        return url if action is None else url + action + '/'

    @classmethod
    def get_list_url(self):
        return 'http://testserver' + reverse('customer-list')


class ProjectFactory(factory.DjangoModelFactory):
    class Meta(object):
        model = models.Project

    name = factory.Sequence(lambda n: 'Proj%s' % n)
    customer = factory.SubFactory(CustomerFactory)

    @classmethod
    def get_url(cls, project=None, action=None):
        if project is None:
            project = ProjectFactory()
        url = 'http://testserver' + reverse('project-detail', kwargs={'uuid': project.uuid})
        return url if action is None else url + action + '/'

    @classmethod
    def get_list_url(cls):
        return 'http://testserver' + reverse('project-list')


class ProjectPermissionFactory(factory.DjangoModelFactory):
    class Meta(object):
        model = models.ProjectPermission

    project = factory.SubFactory(ProjectFactory)
    user = factory.SubFactory(UserFactory)
    role = models.ProjectRole.ADMINISTRATOR

    @classmethod
    def get_url(cls, permission=None, action=None):
        if permission is None:
            permission = ProjectPermissionFactory()
        url = 'http://testserver' + reverse('project_permission-detail', kwargs={'pk': permission.pk})
        return url if action is None else url + action + '/'

    @classmethod
    def get_list_url(self):
        return 'http://testserver' + reverse('project_permission-list')


class CustomerPermissionFactory(factory.DjangoModelFactory):
    class Meta(object):
        model = models.CustomerPermission

    customer = factory.SubFactory(CustomerFactory)
    user = factory.SubFactory(UserFactory)
    role = models.CustomerRole.OWNER

    @classmethod
    def get_url(cls, permission=None, action=None):
        if permission is None:
            permission = CustomerPermissionFactory()
        url = 'http://testserver' + reverse('customer_permission-detail', kwargs={'pk': permission.pk})
        return url if action is None else url + action + '/'

    @classmethod
    def get_list_url(cls):
        return 'http://testserver' + reverse('customer_permission-list')


class ServiceSettingsFactory(factory.DjangoModelFactory):
    class Meta(object):
        model = models.ServiceSettings

    name = factory.Sequence(lambda n: 'Settings %s' % n)
    state = core_models.StateMixin.States.OK
    shared = False
    type = TestConfig.service_name

    @classmethod
    def get_url(cls, settings=None, action=None):
        if settings is None:
            settings = ServiceSettingsFactory()
        url = 'http://testserver' + reverse('servicesettings-detail', kwargs={'uuid': settings.uuid})
        return url if action is None else url + action + '/'

    @classmethod
    def get_list_url(cls):
        return 'http://testserver' + reverse('servicesettings-list')


class TestServiceFactory(factory.DjangoModelFactory):
    class Meta(object):
        model = test_models.TestService

    settings = factory.SubFactory(ServiceSettingsFactory)
    customer = factory.SubFactory(CustomerFactory)

    @classmethod
    def get_url(cls, service=None, action=None):
        if service is None:
            service = TestServiceFactory()
        url = 'http://testserver' + reverse('test-detail', kwargs={'uuid': service.uuid})
        return url if action is None else url + action + '/'

    @classmethod
    def get_list_url(cls):
        return 'http://testserver' + reverse('test-list')


class TestServiceProjectLinkFactory(factory.DjangoModelFactory):
    class Meta(object):
        model = test_models.TestServiceProjectLink

    service = factory.SubFactory(TestServiceFactory)
    project = factory.LazyAttribute(lambda spl: ProjectFactory(customer=spl.service.customer))

    @classmethod
    def get_url(cls, spl=None, action=None):
        if spl is None:
            spl = TestServiceProjectLinkFactory()
        url = 'http://testserver' + reverse('test-spl-detail', kwargs={'pk': spl.pk})
        return url if action is None else url + action + '/'

    @classmethod
    def get_list_url(cls):
        return 'http://testserver' + reverse('test-spl-list')


class TestNewInstanceFactory(factory.DjangoModelFactory):
    class Meta(object):
        model = test_models.TestNewInstance

    name = factory.Sequence(lambda n: 'instance%s' % n)
    service_project_link = factory.SubFactory(TestServiceProjectLinkFactory)

    @classmethod
    def get_url(cls, instance=None, action=None):
        if instance is None:
            instance = TestNewInstanceFactory()
        url = 'http://testserver' + reverse('test-new-instances-detail', kwargs={'uuid': instance.uuid})
        return url if action is None else url + action + '/'

    @classmethod
    def get_list_url(cls):
        return 'http://testserver' + reverse('test-new-instances-list')


class ServiceCertificationFactory(factory.DjangoModelFactory):
    class Meta(object):
        model = models.ServiceCertification

    name = factory.Sequence(lambda n: 'certification%s' % n)
    link = factory.Sequence(lambda n: 'https://www.test.com/certification%s' % n)
    description = factory.Sequence(lambda n: 'description of the certification%s' % n)

    @classmethod
    def get_url(cls, instance=None, action=None):
        if instance is None:
            instance = ServiceCertificationFactory()
        url = 'http://testserver' + reverse('service-certification-detail', kwargs={'uuid': instance.uuid})
        return url if action is None else url + action + '/'

    @classmethod
    def get_list_url(cls):
        return 'http://testserver' + reverse('service-certification-list')

    @factory.post_generation
    def service_settings(self, create, extracted, **kwargs):
        if not create:
            return

        if extracted:
            for service_settings in extracted:
                self.service_settings.add(service_settings)


class TestSubResourceFactory(factory.DjangoModelFactory):
    class Meta(object):
        model = test_models.TestSubResource



from django.test import TestCase

from waldur_core.core.admin import UserChangeForm
from waldur_core.structure.tests.factories import UserFactory


class UserAdminTest(TestCase):
    def change_user(self, **kwargs):
        user = UserFactory()
        form_for_data = UserChangeForm(instance=user)

        post_data = form_for_data.initial
        post_data.update(kwargs)

        form = UserChangeForm(instance=user, data=post_data)
        form.save()

        user.refresh_from_db()
        return user

    def test_civil_number_is_stripped(self):
        user = self.change_user(civil_number='  NEW_CIVIL_NUMBER  ')
        self.assertEqual(user.civil_number, 'NEW_CIVIL_NUMBER')

    def test_whitspace_civil_number_converts_to_none(self):
        user = self.change_user(civil_number='  ')
        self.assertEqual(user.civil_number, None)

    def test_empty_civil_number_converts_to_none(self):
        user = self.change_user(civil_number='')
        self.assertEqual(user.civil_number, None)



from datetime import timedelta

from ddt import ddt, data
from django.conf import settings
from django.utils import timezone
from rest_framework import test, status

from waldur_core.core.tests.helpers import override_waldur_core_settings
from waldur_core.structure import models as structure_models
from waldur_core.structure.tests import factories as structure_factories
from waldur_core.users import models, tasks
from waldur_core.users.tests import factories


class BaseInvitationTest(test.APITransactionTestCase):

    def setUp(self):
        self.staff = structure_factories.UserFactory(is_staff=True)
        self.customer_owner = structure_factories.UserFactory()
        self.project_admin = structure_factories.UserFactory()
        self.project_manager = structure_factories.UserFactory()
        self.user = structure_factories.UserFactory()

        self.customer = structure_factories.CustomerFactory()
        self.customer.add_user(self.customer_owner, structure_models.CustomerRole.OWNER)

        self.customer_role = structure_models.CustomerRole.OWNER
        self.customer_invitation = factories.CustomerInvitationFactory(
            customer=self.customer, customer_role=self.customer_role)

        self.project = structure_factories.ProjectFactory(customer=self.customer)
        self.project.add_user(self.project_admin, structure_models.ProjectRole.ADMINISTRATOR)
        self.project.add_user(self.project_manager, structure_models.ProjectRole.MANAGER)

        self.project_role = structure_models.ProjectRole.ADMINISTRATOR
        self.project_invitation = factories.ProjectInvitationFactory(
            project=self.project, project_role=self.project_role)


@ddt
class InvitationPermissionApiTest(BaseInvitationTest):

    # List tests
    def test_user_can_list_invitations(self):
        self.client.force_authenticate(user=self.user)
        response = self.client.get(factories.InvitationBaseFactory.get_list_url())
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    # Retrieve tests
    @data('staff', 'customer_owner')
    def test_user_with_access_can_retrieve_project_invitation(self, user):
        self.client.force_authenticate(user=getattr(self, user))
        response = self.client.get(factories.ProjectInvitationFactory.get_url(self.project_invitation))
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @data('project_admin', 'project_manager', 'user')
    def test_user_without_access_cannot_retrieve_project_invitation(self, user):
        self.client.force_authenticate(user=getattr(self, user))
        response = self.client.get(factories.ProjectInvitationFactory.get_url(self.project_invitation))
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    @data('staff', 'customer_owner')
    def test_user_with_access_can_retrieve_customer_invitation(self, user):
        self.client.force_authenticate(user=getattr(self, user))
        response = self.client.get(factories.CustomerInvitationFactory.get_url(self.customer_invitation))
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @data('project_admin', 'project_manager', 'user')
    def test_user_without_access_cannot_retrieve_customer_invitation(self, user):
        self.client.force_authenticate(user=getattr(self, user))
        response = self.client.get(factories.CustomerInvitationFactory.get_url(self.customer_invitation))
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    # Creation tests
    @data('staff', 'customer_owner')
    def test_user_with_access_can_create_project_admin_invitation(self, user):
        self.client.force_authenticate(user=getattr(self, user))
        payload = self._get_valid_project_invitation_payload(
            self.project_invitation, project_role=structure_models.ProjectRole.ADMINISTRATOR)
        response = self.client.post(factories.InvitationBaseFactory.get_list_url(), data=payload)
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

    @data('staff', 'customer_owner')
    def test_user_with_access_can_create_project_manager_invitation(self, user):
        self.client.force_authenticate(user=getattr(self, user))
        payload = self._get_valid_project_invitation_payload(
            self.project_invitation, project_role=structure_models.ProjectRole.MANAGER)
        response = self.client.post(factories.InvitationBaseFactory.get_list_url(), data=payload)
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

    @override_waldur_core_settings(OWNERS_CAN_MANAGE_OWNERS=False)
    def test_owner_can_create_project_manager_invitation_if_settings_are_tweaked(self):
        self.client.force_authenticate(user=self.customer_owner)
        payload = self._get_valid_project_invitation_payload(
            self.project_invitation, project_role=structure_models.ProjectRole.MANAGER)
        response = self.client.post(factories.InvitationBaseFactory.get_list_url(), data=payload)
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

    @override_waldur_core_settings(OWNERS_CAN_MANAGE_OWNERS=True)
    @data('project_admin', 'project_manager', 'user')
    def test_user_without_access_cannot_create_project_invitation(self, user):
        self.client.force_authenticate(user=getattr(self, user))
        payload = self._get_valid_project_invitation_payload(self.project_invitation)
        response = self.client.post(factories.InvitationBaseFactory.get_list_url(), data=payload)
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)
        self.assertEqual(response.data, {'detail': 'You do not have permission to perform this action.'})

    @override_waldur_core_settings(OWNERS_CAN_MANAGE_OWNERS=True)
    @data('staff', 'customer_owner')
    def test_user_with_access_can_create_customer_owner_invitation(self, user):
        self.client.force_authenticate(user=getattr(self, user))
        payload = self._get_valid_customer_invitation_payload(self.customer_invitation)
        response = self.client.post(factories.InvitationBaseFactory.get_list_url(), data=payload)
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

    @override_waldur_core_settings(OWNERS_CAN_MANAGE_OWNERS=False)
    def test_owner_can_not_create_customer_owner_invitation_if_settings_are_tweaked(self):
        self.client.force_authenticate(user=self.customer_owner)
        payload = self._get_valid_customer_invitation_payload(self.customer_invitation)
        response = self.client.post(factories.InvitationBaseFactory.get_list_url(), data=payload)
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @override_waldur_core_settings(OWNERS_CAN_MANAGE_OWNERS=True)
    @data('staff', 'customer_owner')
    def test_user_which_created_invitation_is_stored_in_inviatation(self, user):
        self.client.force_authenticate(user=getattr(self, user))
        payload = self._get_valid_customer_invitation_payload(self.customer_invitation)
        response = self.client.post(factories.InvitationBaseFactory.get_list_url(), data=payload)
        invitation = models.Invitation.objects.get(uuid=response.data['uuid'])
        self.assertEqual(invitation.created_by, getattr(self, user))

    @data('project_admin', 'project_manager', 'user')
    def test_user_without_access_cannot_create_customer_owner_invitation(self, user):
        self.client.force_authenticate(user=getattr(self, user))
        payload = self._get_valid_customer_invitation_payload(self.customer_invitation)
        response = self.client.post(factories.InvitationBaseFactory.get_list_url(), data=payload)
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)
        self.assertEqual(response.data, {'detail': 'You do not have permission to perform this action.'})

    @data('staff', 'customer_owner')
    def test_user_with_access_can_cancel_project_invitation(self, user):
        self.client.force_authenticate(user=getattr(self, user))
        response = self.client.post(factories.ProjectInvitationFactory.get_url(self.project_invitation,
                                                                               action='cancel'))
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.project_invitation.refresh_from_db()
        self.assertEqual(self.project_invitation.state, models.Invitation.State.CANCELED)

    @data('project_admin', 'project_manager', 'user')
    def test_user_without_access_cannot_cancel_project_invitation(self, user):
        self.client.force_authenticate(user=getattr(self, user))
        response = self.client.post(factories.ProjectInvitationFactory.get_url(self.project_invitation,
                                                                               action='cancel'))
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    @override_waldur_core_settings(OWNERS_CAN_MANAGE_OWNERS=True)
    @data('staff', 'customer_owner')
    def test_user_with_access_can_cancel_customer_invitation(self, user):
        self.client.force_authenticate(user=getattr(self, user))
        response = self.client.post(factories.CustomerInvitationFactory.get_url(self.customer_invitation,
                                                                                action='cancel'))
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.customer_invitation.refresh_from_db()
        self.assertEqual(self.customer_invitation.state, models.Invitation.State.CANCELED)

    @override_waldur_core_settings(OWNERS_CAN_MANAGE_OWNERS=False)
    def test_owner_can_not_cancel_customer_invitation_if_settings_are_tweaked(self):
        self.client.force_authenticate(user=self.customer_owner)
        response = self.client.post(factories.CustomerInvitationFactory.get_url(self.customer_invitation,
                                                                                action='cancel'))
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @override_waldur_core_settings(OWNERS_CAN_MANAGE_OWNERS=True)
    @data('staff', 'customer_owner')
    def test_user_with_access_can_send_customer_invitation(self, user):
        self.client.force_authenticate(user=getattr(self, user))
        response = self.client.post(factories.CustomerInvitationFactory.get_url(self.customer_invitation,
                                                                                action='send'))
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @override_waldur_core_settings(OWNERS_CAN_MANAGE_OWNERS=False)
    def test_owner_can_not_send_customer_invitation_if_settings_are_tweaked(self):
        self.client.force_authenticate(user=self.customer_owner)
        response = self.client.post(factories.CustomerInvitationFactory.get_url(self.customer_invitation,
                                                                                action='send'))
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @data('staff', 'customer_owner')
    def test_user_with_access_can_send_project_invitation(self, user):
        self.client.force_authenticate(user=getattr(self, user))
        response = self.client.post(factories.ProjectInvitationFactory.get_url(self.project_invitation,
                                                                               action='send'))
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_owner_can_send_project_invitation_if_settings_are_tweaked(self):
        self.client.force_authenticate(user=self.customer_owner)
        response = self.client.post(factories.ProjectInvitationFactory.get_url(self.project_invitation,
                                                                               action='send'))
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @data('project_admin', 'project_manager', 'user')
    def test_user_without_access_cannot_send_project_invitation(self, user):
        self.client.force_authenticate(user=getattr(self, user))
        response = self.client.post(factories.ProjectInvitationFactory.get_url(self.project_invitation,
                                                                               action='send'))
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    # API tests
    def test_user_cannot_create_invitation_with_invalid_link_template(self):
        self.client.force_authenticate(user=self.staff)
        payload = self._get_valid_project_invitation_payload(self.project_invitation)
        payload['link_template'] = '/invalid/link'
        response = self.client.post(factories.ProjectInvitationFactory.get_list_url(), data=payload)
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertEqual(response.data, {'link_template': ["Link template must include '{uuid}' parameter."]})

    def test_user_cannot_create_invitation_for_project_and_customer_simultaneously(self):
        self.client.force_authenticate(user=self.staff)
        payload = self._get_valid_project_invitation_payload(self.project_invitation)
        customer_payload = self._get_valid_customer_invitation_payload(self.customer_invitation)
        payload['customer'] = customer_payload['customer']
        payload['customer_role'] = customer_payload['customer_role']

        response = self.client.post(factories.InvitationBaseFactory.get_list_url(), data=payload)
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertEqual(response.data,
                         {'non_field_errors': ["Cannot create invitation to project and customer simultaneously."]})

    def test_user_cannot_create_invitation_without_customer_or_project(self):
        self.client.force_authenticate(user=self.staff)
        payload = self._get_valid_project_invitation_payload(self.project_invitation)
        payload.pop('project')

        response = self.client.post(factories.InvitationBaseFactory.get_list_url(), data=payload)
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertEqual(response.data, {'non_field_errors': ["Customer or project must be provided."]})

    def test_user_cannot_create_project_invitation_without_project_role(self):
        self.client.force_authenticate(user=self.staff)
        payload = self._get_valid_project_invitation_payload(self.project_invitation)
        payload.pop('project_role')

        response = self.client.post(factories.InvitationBaseFactory.get_list_url(), data=payload)
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertEqual(response.data, {'project_role': ["Project and its role must be provided."]})

    def test_user_cannot_create_customer_invitation_without_customer_role(self):
        self.client.force_authenticate(user=self.staff)
        payload = self._get_valid_customer_invitation_payload(self.customer_invitation)
        payload.pop('customer_role')

        response = self.client.post(factories.InvitationBaseFactory.get_list_url(), data=payload)
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertEqual(response.data, {'customer_role': ["Customer and its role must be provided."]})

    def test_user_can_create_invitation_for_existing_user(self):
        self.client.force_authenticate(user=self.staff)
        email = 'test@example.com'
        structure_factories.UserFactory(email=email)
        payload = self._get_valid_project_invitation_payload(self.project_invitation)
        payload['email'] = email

        response = self.client.post(factories.InvitationBaseFactory.get_list_url(), data=payload)
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

    def test_invitation_is_canceled_after_expiration_date(self):
        waldur_section = settings.WALDUR_CORE.copy()
        waldur_section['INVITATION_LIFETIME'] = timedelta(weeks=1)

        with self.settings(WALDUR_CORE=waldur_section):
            invitation = factories.ProjectInvitationFactory(created=timezone.now() - timedelta(weeks=1))
            tasks.cancel_expired_invitations()

        self.assertEqual(models.Invitation.objects.get(uuid=invitation.uuid).state, models.Invitation.State.EXPIRED)

    def test_filtering_by_customer_uuid_includes_project_invitations_for_that_customer_too(self):
        self.client.force_authenticate(user=self.staff)
        response = self.client.get(factories.InvitationBaseFactory.get_list_url(), {
            'customer': self.customer.uuid.hex
        })
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(len(response.data), 2)

    def test_filtering_by_customer_url_includes_project_invitations_for_that_customer_too(self):
        self.client.force_authenticate(user=self.staff)
        response = self.client.get(factories.InvitationBaseFactory.get_list_url(), {
            'customer_url': structure_factories.CustomerFactory.get_url(self.customer)
        })
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(len(response.data), 2)

    def test_filtering_by_another_customer_does_not_includes_project_invitations_for_initial_customer(self):
        other_customer = structure_factories.CustomerFactory()
        self.client.force_authenticate(user=self.staff)
        response = self.client.get(factories.InvitationBaseFactory.get_list_url(), {
            'customer': other_customer.uuid.hex
        })
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(len(response.data), 0)

    # Helper methods
    def _get_valid_project_invitation_payload(self, invitation=None, project_role=None):
        invitation = invitation or factories.ProjectInvitationFactory.build()
        return {
            'email': invitation.email,
            'link_template': invitation.link_template,
            'project': structure_factories.ProjectFactory.get_url(invitation.project),
            'project_role': project_role or structure_models.ProjectRole.ADMINISTRATOR,
        }

    def _get_valid_customer_invitation_payload(self, invitation=None, customer_role=None):
        invitation = invitation or factories.CustomerInvitationFactory.build()
        return {
            'email': invitation.email,
            'link_template': invitation.link_template,
            'customer': structure_factories.CustomerFactory.get_url(invitation.customer),
            'customer_role': customer_role or structure_models.CustomerRole.OWNER,
        }


class InvitationAcceptTest(BaseInvitationTest):

    def test_authenticated_user_can_accept_project_invitation(self):
        self.client.force_authenticate(user=self.user)
        response = self.client.post(factories.ProjectInvitationFactory.get_url(
            self.project_invitation, action='accept'))

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.project_invitation.refresh_from_db()
        self.assertEqual(self.project_invitation.state, models.Invitation.State.ACCEPTED)
        self.assertTrue(self.project.has_user(self.user, self.project_invitation.project_role))

    def test_authenticated_user_can_accept_customer_invitation(self):
        self.client.force_authenticate(user=self.user)
        response = self.client.post(factories.CustomerInvitationFactory.get_url(
            self.customer_invitation, action='accept'))

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.customer_invitation.refresh_from_db()
        self.assertEqual(self.customer_invitation.state, models.Invitation.State.ACCEPTED)
        self.assertTrue(self.customer.has_user(self.user, self.customer_invitation.customer_role))

    def test_user_with_invalid_civil_number_cannot_accept_invitation(self):
        customer_invitation = factories.CustomerInvitationFactory(
            customer=self.customer, customer_role=self.customer_role, civil_number='123456789')
        self.client.force_authenticate(user=self.user)
        response = self.client.post(factories.CustomerInvitationFactory.get_url(customer_invitation, action='accept'))

        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertEqual(response.data, ['User has an invalid civil number.'])

    def test_user_which_already_has_role_within_customer_cannot_accept_invitation(self):
        customer_invitation = factories.CustomerInvitationFactory(
            customer=self.customer, customer_role=self.customer_role)
        self.client.force_authenticate(user=self.user)
        self.customer.add_user(self.user, customer_invitation.customer_role)
        response = self.client.post(factories.CustomerInvitationFactory.get_url(customer_invitation, action='accept'))

        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertEqual(response.data, ['User already has role within this customer.'])

    def test_user_which_already_has_role_within_project_cannot_accept_invitation(self):
        project_invitation = factories.ProjectInvitationFactory(
            project=self.project, project_role=self.project_role)
        self.client.force_authenticate(user=self.user)
        self.project.add_user(self.user, project_invitation.project_role)
        response = self.client.post(factories.ProjectInvitationFactory.get_url(project_invitation, action='accept'))

        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertEqual(response.data, ['User already has role within this project.'])

    def test_user_which_created_invitation_is_stored_in_permission(self):
        invitation = factories.CustomerInvitationFactory(created_by=self.customer_owner)
        self.client.force_authenticate(user=self.user)
        self.client.post(factories.CustomerInvitationFactory.get_url(invitation, action='accept'))
        permission = structure_models.CustomerPermission.objects.get(user=self.user, customer=invitation.customer)
        self.assertEqual(permission.created_by, self.customer_owner)

    def test_user_can_rewrite_his_email_on_invitation_accept(self):
        invitation = factories.CustomerInvitationFactory(created_by=self.customer_owner, email='invitation@i.ua')
        self.client.force_authenticate(user=self.user)

        self.client.post(
            factories.CustomerInvitationFactory.get_url(invitation, action='accept'), {'replace_email': True})

        self.assertEqual(self.user.email, invitation.email)

    @override_waldur_core_settings(VALIDATE_INVITATION_EMAIL=True)
    def test_user_can_not_rewrite_his_email_on_acceptance_if_validation_of_emails_is_on(self):
        invitation = factories.CustomerInvitationFactory(created_by=self.customer_owner, email='invitation@i.ua')
        self.client.force_authenticate(user=self.user)
        url = factories.CustomerInvitationFactory.get_url(invitation, action='accept')

        response = self.client.post(url, {'replace_email': True})

        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.user.refresh_from_db()
        self.assertNotEqual(self.user.email, invitation.email)

    @override_waldur_core_settings(VALIDATE_INVITATION_EMAIL=False)
    def test_user_can_rewrite_his_email_on_acceptance_if_validation_of_emails_is_off(self):
        invitation = factories.CustomerInvitationFactory(created_by=self.customer_owner, email=self.user.email)
        self.client.force_authenticate(user=self.user)
        url = factories.CustomerInvitationFactory.get_url(invitation, action='accept')

        response = self.client.post(url, {'replace_email': True})

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.user.refresh_from_db()
        self.assertEqual(self.user.email, invitation.email)

    @override_waldur_core_settings(VALIDATE_INVITATION_EMAIL=True)
    def test_user_can_accept_invitation_if_emails_match_and_validation_of_emails_is_on(self):
        invitation = factories.CustomerInvitationFactory(created_by=self.customer_owner, email=self.user.email)
        self.client.force_authenticate(user=self.user)
        url = factories.CustomerInvitationFactory.get_url(invitation, action='accept')

        response = self.client.post(url, {'replace_email': True})

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.user.refresh_from_db()
        self.assertEqual(self.user.email, invitation.email)



import unittest

import mock

from django.conf import settings
from django.test import override_settings
from rest_framework import test
from rest_framework import status

from waldur_core.structure import models as structure_models
from waldur_core.structure.tests import factories as structure_factories

from . import factories
from .. import utils
from ..loggers import EventLogger, event_logger


def override_elasticsearch_settings(**kwargs):
    waldur_settings = settings.WALDUR_CORE.copy()
    waldur_settings['ELASTICSEARCH'] = {
        'username': 'username',
        'password': 'password',
        'host': 'example.com',
        'port': '9999',
        'protocol': 'https',
    }
    return override_settings(WALDUR_CORE=waldur_settings, **kwargs)


@override_elasticsearch_settings()
class BaseEventsApiTest(test.APITransactionTestCase):
    def setUp(self):
        self.es_patcher = mock.patch('waldur_core.logging.elasticsearch_client.Elasticsearch')
        self.mocked_es = self.es_patcher.start()
        self.mocked_es().search.return_value = {'hits': {'total': 0, 'hits': []}}

    def tearDown(self):
        self.es_patcher.stop()

    def get_term(self, name):
        call_args = self.mocked_es().search.call_args[-1]
        query = call_args['body']['query']['bool']
        if name in query:
            return query[name][-1]['terms']

    @property
    def must_terms(self):
        return self.get_term('must')

    @property
    def must_not_terms(self):
        return self.get_term('must_not')


class DebugEventLogger(EventLogger):

    class Meta:
        event_types = (
            'debug_started',
            'debug_succeeded',
            'debug_failed',
        )
        event_groups = {
            'debug_only': event_types,
        }


class ExtraEventLogger(EventLogger):

    class Meta:
        event_types = (
            'update_started',
            'update_succeeded',
            'update_failed',
        )
        event_groups = {
            'update': event_types
        }


class UserEventLogger(EventLogger):

    class Meta:
        event_types = (
            'user_created',
            'user_deleted',
        )
        event_groups = {
            'user': event_types
        }


class EventGetTest(BaseEventsApiTest):

    def setUp(self):
        super(EventGetTest, self).setUp()
        staff = structure_factories.UserFactory(is_staff=True)
        self.client.force_authenticate(user=staff)

        self.old_loggers = event_logger.__dict__.copy()
        event_logger.unregister_all()
        event_logger.register('debug_logger', DebugEventLogger)
        event_logger.register('extra_logger', ExtraEventLogger)
        event_logger.register('user_logger', UserEventLogger)

    def tearDown(self):
        event_logger.__dict__ = self.old_loggers

    @override_elasticsearch_settings(DEBUG=True)
    def test_debug_events_are_not_filtered_out_in_debug_mode(self):
        self.get_events()
        self.assertIsNone(self.must_not_terms)

    @override_elasticsearch_settings(DEBUG=False)
    def test_debug_events_are_filtered_out_in_production_mode(self):
        self.get_events()

        self.assertItemsEqual(self.must_not_terms['event_type'], [
            'debug_started',
            'debug_succeeded',
            'debug_failed',
        ])

    @override_elasticsearch_settings(DEBUG=False)
    def test_extra_and_debug_events_combined(self):
        self.get_events({'exclude_extra': True})
        self.assertItemsEqual(self.must_not_terms['event_type'], [
            'debug_started',
            'debug_succeeded',
            'debug_failed',
            'update_started',
            'update_succeeded',
            'update_failed',
        ])

    @override_elasticsearch_settings(DEBUG=False)
    def test_features_and_debug_events_combined(self):
        self.get_events({'exclude_features': ['user']})
        self.assertItemsEqual(self.must_not_terms['event_type'], [
            'debug_started',
            'debug_succeeded',
            'debug_failed',
            'user_created',
            'user_deleted',
        ])

    def get_events(self, params=None):
        return self.client.get(factories.EventFactory.get_list_url(), params)


class ScopeTypeTest(BaseEventsApiTest):
    def _get_events_by_scope_type(self, model):
        url = factories.EventFactory.get_list_url()
        scope_type = utils.get_reverse_scope_types_mapping()[model]
        return self.client.get(url, {'scope_type': scope_type})

    def test_staff_can_see_any_customers_events(self):
        staff = structure_factories.UserFactory(is_staff=True)
        self.client.force_authenticate(user=staff)
        customer = structure_factories.CustomerFactory()

        self._get_events_by_scope_type(structure_models.Customer)
        self.assertEqual(self.must_terms, {'customer_uuid': [customer.uuid.hex]})

    def test_owner_can_see_only_customer_events(self):
        structure_factories.CustomerFactory()

        customer = structure_factories.CustomerFactory()
        owner = structure_factories.UserFactory()
        customer.add_user(owner, structure_models.CustomerRole.OWNER)

        self.client.force_authenticate(user=owner)
        self._get_events_by_scope_type(structure_models.Customer)
        self.assertEqual(self.must_terms, {'customer_uuid': [customer.uuid.hex]})

    def test_project_administrator_can_see_his_project_events(self):
        project = structure_factories.ProjectFactory()
        admin = structure_factories.UserFactory()
        project.add_user(admin, structure_models.ProjectRole.ADMINISTRATOR)

        self.client.force_authenticate(user=admin)
        self._get_events_by_scope_type(structure_models.Project)
        self.assertEqual(self.must_terms, {'project_uuid': [project.uuid.hex]})

    def test_project_administrator_cannot_see_other_projects_events(self):
        user = structure_factories.UserFactory()

        structure_factories.ProjectFactory()

        self.client.force_authenticate(user=user)
        self._get_events_by_scope_type(structure_models.Project)
        self.assertEqual(self.must_terms, {'project_uuid': []})

    def test_project_administrator_cannot_see_related_customer_events(self):
        project = structure_factories.ProjectFactory()
        admin = structure_factories.UserFactory()
        project.add_user(admin, structure_models.ProjectRole.ADMINISTRATOR)

        self.client.force_authenticate(user=admin)
        self._get_events_by_scope_type(structure_models.Customer)
        self.assertEqual(self.must_terms, {'customer_uuid': []})


class ScopeTest(BaseEventsApiTest):
    def _get_events_by_scope(self, scope):
        url = factories.EventFactory.get_list_url()
        return self.client.get(url, {'scope': scope})

    @unittest.skip('NC-1485')
    def test_project_administrator_cannot_see_related_customer_events(self):
        project = structure_factories.ProjectFactory()
        admin = structure_factories.UserFactory()
        project.add_user(admin, structure_models.ProjectRole.ADMINISTRATOR)

        self.client.force_authenticate(user=admin)
        response = self._get_events_by_scope(structure_factories.CustomerFactory.get_url(project.customer))
        self.assertFalse(self.mocked_es().search.called)
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    def test_customer_owner_can_see_his_customer_events(self):
        customer = structure_factories.CustomerFactory()
        owner = structure_factories.UserFactory()
        customer.add_user(owner, structure_models.CustomerRole.OWNER)

        self.client.force_authenticate(user=owner)
        self._get_events_by_scope(structure_factories.CustomerFactory.get_url(customer))
        self.assertEqual(self.must_terms, {'customer_uuid.keyword': [customer.uuid.hex]})



import logging

from django.contrib import auth
from django.core.cache import cache
from django.db.models import ProtectedError
from django.utils import timezone
from django.utils.encoding import force_text
from django.utils.translation import ugettext_lazy as _

from rest_framework import status, mixins as rf_mixins, viewsets, permissions as rf_permissions, exceptions
from rest_framework.authtoken.models import Token
from rest_framework.decorators import api_view, permission_classes
from rest_framework.response import Response
from rest_framework.views import APIView
from rest_framework.views import exception_handler as rf_exception_handler

from waldur_core import __version__
from waldur_core.core import permissions
from waldur_core.core.exceptions import IncorrectStateException
from waldur_core.core.serializers import AuthTokenSerializer
from waldur_core.logging.loggers import event_logger


logger = logging.getLogger(__name__)


class RefreshTokenMixin(object):
    """
    This mixin is used in both password and social auth (implemented via plugin).
    Mixin allows to create new token if it does not exist yet or if it has already expired.
    Token is refreshed if it has not expired yet.
    """
    def refresh_token(self, user):
        token, created = Token.objects.get_or_create(user=user)

        if user.token_lifetime:
            lifetime = timezone.timedelta(seconds=user.token_lifetime)

            if token.created < timezone.now() - lifetime:
                token.delete()
                token = Token.objects.create(user=user)
                created = True

        if not created:
            token.created = timezone.now()
            token.save(update_fields=['created'])

        return token


class ObtainAuthToken(RefreshTokenMixin, APIView):
    """
    Api view loosely based on DRF's default ObtainAuthToken,
    but with the responses formats and status codes aligned with BasicAuthentication behavior.

    Valid request example:

    .. code-block:: http

        POST /api-auth/password/ HTTP/1.1
        Accept: application/json
        Content-Type: application/json
        Host: example.com

        {
            "username": "alice",
            "password": "$ecr3t"
        }

    Success response example:

    .. code-block:: http

        HTTP/1.0 200 OK
        Allow: POST, OPTIONS
        Content-Type: application/json
        Vary: Accept, Cookie

        {
            "token": "c84d653b9ec92c6cbac41c706593e66f567a7fa4"
        }

    Field validation failure response example:

    .. code-block:: http

        HTTP/1.0 401 UNAUTHORIZED
        Allow: POST, OPTIONS
        Content-Type: application/json

        {
            "password": ["This field is required."]
        }

    Invalid credentials failure response example:

    .. code-block:: http

        HTTP/1.0 401 UNAUTHORIZED
        Allow: POST, OPTIONS
        Content-Type: application/json

        {
            "detail": "Invalid username/password"
        }
    """
    throttle_classes = ()
    permission_classes = ()
    serializer_class = AuthTokenSerializer

    def post(self, request):
        serializer = self.serializer_class(data=request.data)
        serializer.is_valid(raise_exception=True)

        username = serializer.validated_data['username']

        source_ip = request.META.get('REMOTE_ADDR')
        auth_failure_key = 'LOGIN_FAILURES_OF_%s_AT_%s' % (username, source_ip)
        auth_failures = cache.get(auth_failure_key) or 0
        lockout_time_in_mins = 10

        if auth_failures >= 4:
            logger.debug('Not returning auth token: '
                         'username %s from %s is locked out' % (username, source_ip))
            return Response(
                data={'detail': _('Username is locked out. Try in %s minutes.') % lockout_time_in_mins},
                status=status.HTTP_401_UNAUTHORIZED,
            )
        user = auth.authenticate(
            request=request,
            username=username,
            password=serializer.validated_data['password'],
        )

        if not user:
            logger.debug('Not returning auth token: '
                         'user %s does not exist', username)
            cache.set(auth_failure_key, auth_failures + 1, lockout_time_in_mins * 60)
            event_logger.auth.info(
                'User {username} failed to authenticate with username and password.',
                event_type='auth_login_failed_with_username',
                event_context={'username': username})

            return Response(
                data={'detail': _('Invalid username/password.')},
                status=status.HTTP_401_UNAUTHORIZED,
            )
        else:
            cache.delete(auth_failure_key)

        if not user.is_active:
            logger.debug('Not returning auth token: '
                         'user %s is disabled', username)
            return Response(
                data={'detail': _('User account is disabled.')},
                status=status.HTTP_401_UNAUTHORIZED,
            )

        token = self.refresh_token(user)

        logger.debug('Returning token for successful login of user %s', user)
        event_logger.auth.info(
            'User {user_username} with full name {user_full_name} '
            'authenticated successfully with username and password.',
            event_type='auth_logged_in_with_username',
            event_context={'user': user})

        return Response({'token': token.key})


obtain_auth_token = ObtainAuthToken.as_view()


@api_view(['GET'])
@permission_classes((rf_permissions.AllowAny, ))
def version_detail(request):
    """Retrieve version of the application"""

    return Response({'version': __version__})


# noinspection PyProtectedMember
def exception_handler(exc, context):
    if isinstance(exc, ProtectedError):
        dependent_meta = exc.protected_objects.model._meta

        try:
            # This exception should be raised from a viewset
            instance_meta = context['view'].get_queryset().model._meta
        except (AttributeError, KeyError):
            # Fallback, when instance being deleted cannot be inferred
            instance_name = 'object'
        else:
            instance_name = force_text(instance_meta.verbose_name)

        detail = _('Cannot delete {instance_name} with existing {dependant_objects}').format(
            instance_name=instance_name,
            dependant_objects=force_text(dependent_meta.verbose_name_plural),
        )

        # We substitute exception here to get consistent representation
        # for both ProtectError and manually raised IncorrectStateException
        exc = IncorrectStateException(detail=detail)

    return rf_exception_handler(exc, context)


class ProtectedViewSet(rf_mixins.CreateModelMixin,
                       rf_mixins.RetrieveModelMixin,
                       rf_mixins.ListModelMixin,
                       viewsets.GenericViewSet):
    """ All default operations except update and delete """
    pass


class ActionsViewSet(viewsets.ModelViewSet):
    """
    Treats all endpoint actions in the same way.

    1. Allow to define separate serializers for each action:

        def action(self, request, *args, **kwargs):
            serializer = self.get_serializer(...)
            ...

        action_serializer_class = ActionSerializer

    2. Allow to define validators for detail actions:

        def state_is_ok(obj):
            if obj.state != 'ok':
                raise IncorrectStateException('Instance should be in state OK.')

        @decorators.detail_route()
        def action(self, request, *args, **kwargs):
            ...

        action_validators = [state_is_ok]

    3. Allow to define permissions checks for all actions or each action
       separately. Check ActionPermissionsBackend for more details.

    4. To avoid dancing around mixins - allow disabling actions:

        class MyView(ActionsViewSet):
            disabled_actions = ['create']  # error 405 will be returned on POST request
    """
    disabled_actions = []
    permission_classes = (rf_permissions.IsAuthenticated, permissions.ActionsPermission)

    def get_serializer_class(self):
        default_serializer_class = super(ActionsViewSet, self).get_serializer_class()
        if self.action is None:
            return default_serializer_class
        return getattr(self, self.action + '_serializer_class', default_serializer_class)

    def initial(self, request, *args, **kwargs):
        super(ActionsViewSet, self).initial(request, *args, **kwargs)
        if self.action is None:  # disable all checks if user tries to reach unsupported action
            return
        # check if action is allowed
        if self.action in getattr(self, 'disabled_actions', []):
            raise exceptions.MethodNotAllowed(method=request.method)
        if self.action != 'metadata':
            self.validate_object_action(self.action)

    def validate_object_action(self, action_name, obj=None):
        """ Execute validation for actions that are related to particular object """
        action_method = getattr(self, action_name)
        if not getattr(action_method, 'detail', False) and action_name not in ('update', 'partial_update', 'destroy'):
            # DRF does not add flag 'detail' to update and delete actions, however they execute operation with
            # particular object. We need to enable validation for them too.
            return
        validators = getattr(self, action_name + '_validators', [])
        for validator in validators:
            validator(obj or self.get_object())


class ReadOnlyActionsViewSet(ActionsViewSet):
    disabled_actions = ['create', 'update', 'partial_update', 'destroy']



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('users', '0002_invitation_error_message'),
    ]

    operations = [
        migrations.AddField(
            model_name='invitation',
            name='civil_number',
            field=models.CharField(help_text='Civil number of invited user. If civil number is not defined any user can accept invitation.', max_length=50, blank=True),
        ),
    ]
